@@ -75,11 +75,17 @@ FOptionContainer::~FOptionContainer()
 	if (grey_url_flag) o.lm.deRefList(grey_url_list);
 	if (banned_regexpurl_flag) o.lm.deRefList(banned_regexpurl_list);
 	if (exception_regexpurl_flag) o.lm.deRefList(exception_regexpurl_list);
+	if (banned_regexpheader_flag) o.lm.deRefList(banned_regexpheader_list);
 	if (content_regexp_flag) o.lm.deRefList(content_regexp_list);
 	if (url_regexp_flag) o.lm.deRefList(url_regexp_list);
+	if (header_regexp_flag) o.lm.deRefList(header_regexp_list);
 	if (exception_extension_flag) o.lm.deRefList(exception_extension_list);
 	if (exception_mimetype_flag) o.lm.deRefList(exception_mimetype_list);
 	if (exception_file_site_flag) o.lm.deRefList(exception_file_site_list);
+	if (exception_file_url_flag) o.lm.deRefList(exception_file_url_list);
+	if (log_site_flag) o.lm.deRefList(log_site_list);
+	if (log_url_flag) o.lm.deRefList(log_url_list);
+	if (log_regexpurl_flag) o.lm.deRefList(log_regexpurl_list);
 	delete banned_page;
 }
 
@@ -96,11 +102,17 @@ void FOptionContainer::reset()
 	if (grey_url_flag) o.lm.deRefList(grey_url_list);
 	if (banned_regexpurl_flag) o.lm.deRefList(banned_regexpurl_list);
 	if (exception_regexpurl_flag) o.lm.deRefList(exception_regexpurl_list);
+	if (banned_regexpheader_flag) o.lm.deRefList(banned_regexpheader_list);
 	if (content_regexp_flag) o.lm.deRefList(content_regexp_list);
 	if (url_regexp_flag) o.lm.deRefList(url_regexp_list);
+	if (header_regexp_flag) o.lm.deRefList(header_regexp_list);
 	if (exception_extension_flag) o.lm.deRefList(exception_extension_list);
 	if (exception_mimetype_flag) o.lm.deRefList(exception_mimetype_list);
 	if (exception_file_site_flag) o.lm.deRefList(exception_file_site_list);
+	if (exception_file_url_flag) o.lm.deRefList(exception_file_url_list);
+	if (log_site_flag) o.lm.deRefList(log_site_list);
+	if (log_url_flag) o.lm.deRefList(log_url_list);
+	if (log_regexpurl_flag) o.lm.deRefList(log_regexpurl_list);
 	banned_phrase_flag = false;
 	exception_site_flag = false;
 	exception_url_flag = false;
@@ -112,28 +124,38 @@ void FOptionContainer::reset()
 	grey_url_flag = false;
 	banned_regexpurl_flag = false;
 	exception_regexpurl_flag = false;
+	banned_regexpheader_flag = false;
 	content_regexp_flag = false;
 	url_regexp_flag = false;
+	header_regexp_flag = false;
 	exception_extension_flag = false;
 	exception_mimetype_flag = false;
 	exception_file_site_flag = false;
+	exception_file_url_flag = false;
+	log_site_flag = false;
+	log_url_flag = false;
+	log_regexpurl_flag = false;
 	block_downloads = false;
-	blanketblock = false;
-	blanket_ip_block = false;
-	blanketsslblock = false;
-	blanketssl_ip_block = false;
 	banned_phrase_list_index.clear();
 	conffile.clear();
 	content_regexp_list_comp.clear();
 	content_regexp_list_rep.clear();
 	url_regexp_list_comp.clear();
 	url_regexp_list_rep.clear();
+	header_regexp_list_comp.clear();
+	header_regexp_list_rep.clear();
 	banned_regexpurl_list_comp.clear();
 	banned_regexpurl_list_source.clear();
 	banned_regexpurl_list_ref.clear();
 	exception_regexpurl_list_comp.clear();
 	exception_regexpurl_list_source.clear();
 	exception_regexpurl_list_ref.clear();
+	banned_regexpheader_list_comp.clear();
+	banned_regexpheader_list_source.clear();
+	banned_regexpheader_list_ref.clear();
+	log_regexpurl_list_comp.clear();
+	log_regexpurl_list_source.clear();
+	log_regexpurl_list_ref.clear();
 	delete banned_page;
 	banned_page = NULL;
 }
@@ -165,7 +187,7 @@ bool FOptionContainer::readFile(const char *filename, unsigned int* whichlist, b
 			(*o.lm.l[(*whichlist)]).doSort(true);
 		else
 			(*o.lm.l[(*whichlist)]).doSort(false);
-		if (cache && (createlistcachefiles == 1)) {
+		if (cache && createlistcachefiles) {
 			if (!(*o.lm.l[(*whichlist)]).createCacheFile()) {
 				return false;
 			}
@@ -214,53 +236,53 @@ bool FOptionContainer::read(const char *filename)
 #endif
 
 		if (findoptionS("deepurlanalysis") == "on") {
-			deep_url_analysis = 1;
+			deep_url_analysis = true;
 		} else {
-			deep_url_analysis = 0;
+			deep_url_analysis = false;
 		}
 
 		if (findoptionS("disablecontentscan") == "on") {
-			disable_content_scan = 1;
+			disable_content_scan = true;
 		} else {
-			disable_content_scan = 0;
+			disable_content_scan = false;
 		}
 
 #ifdef __EMAIL
 		// Email notification patch by J. Gauthier
 		if (findoptionS("usesmtp") == "on") {
-			use_smtp = 1;
+			use_smtp = true;
 		} else {
-			use_smtp = 0;
+			use_smtp = false;
 		}
 
 		if (findoptionS("thresholdbyuser") == "on") {
-			byuser = 1;
+			byuser = true;
 		} else {
-			byuser = 0;
+			byuser = false;
 		}	   
 
 		if (findoptionS("notifyav") == "on") {
-			if (use_smtp == 0) {
+			if (!use_smtp) {
 				if (!is_daemonised)
 					std::cerr << "notifyav cannot be on while usesmtp is off." << std::endl;
 				syslog(LOG_ERR, "notifyav cannot be on while usesmtp is off.");
 				return false;
 			}
-			notifyav = 1;
+			notifyav = true;
 		} else {
-			notifyav = 0;
+			notifyav = false;
 		}
 
 		if (findoptionS("notifycontent") == "on") {
-			if (use_smtp == 0) {
+			if (!use_smtp) {
 				if (!is_daemonised)
 					std::cerr << "notifycontent cannot be on while usesmtp is off." << std::endl;
 				syslog(LOG_ERR, "notifycontent cannot be on while usesmtp is off.");
 				return false;
 			}
-			notifycontent = 1;
+			notifycontent = true;
 		} else {
-			notifycontent = 0;
+			notifycontent = false;
 		}
 
 		violations = findoptionI("violations");
@@ -280,7 +302,7 @@ bool FOptionContainer::read(const char *filename)
 
 		contentadmin = findoptionS("contentadmin");
 		if (contentadmin.length()==0) {
-			if (use_smtp == 1) {		   
+			if (use_smtp) {		   
 				if (!is_daemonised)		   
 					std::cerr << "contentadmin cannot be blank while usesmtp is on." << std::endl;
 				syslog(LOG_ERR, "contentadmin cannot be blank while usesmtp is on.");
@@ -290,7 +312,7 @@ bool FOptionContainer::read(const char *filename)
 
 		mailfrom = findoptionS("mailfrom");
 		if (mailfrom.length()==0) {
-			if (use_smtp == 1) {
+			if (use_smtp) {
 				if (!is_daemonised)		   
 					std::cerr << "mailfrom cannot be blank while usesmtp is on." << std::endl;
 				syslog(LOG_ERR, "mailfrom cannot be blank while usesmtp is on.");
@@ -399,16 +421,16 @@ bool FOptionContainer::read(const char *filename)
 			// deque.  They are only seperate files for clarity.
 
 			if (findoptionS("enablepics") == "on") {
-				enable_PICS = 1;
+				enable_PICS = true;
 			} else {
-				enable_PICS = 0;
+				enable_PICS = false;
 			}
 
 			if (findoptionS("blockdownloads") == "on") {
 				block_downloads = true;
 			}
 
-			if (enable_PICS == 1) {
+			if (enable_PICS) {
 				linebuffer = findoptionS("picsfile");
 				ifstream picsfiles(linebuffer.c_str(), ios::in);  // pics file
 				if (!picsfiles.good()) {
@@ -458,15 +480,21 @@ bool FOptionContainer::read(const char *filename)
 			std::string grey_url_list_location(findoptionS("greyurllist"));
 			std::string banned_regexpurl_list_location(findoptionS("bannedregexpurllist"));
 			std::string exception_regexpurl_list_location(findoptionS("exceptionregexpurllist"));
+			std::string banned_regexpheader_list_location(findoptionS("bannedregexpheaderlist"));
 			std::string content_regexp_list_location(findoptionS("contentregexplist"));
 			std::string url_regexp_list_location(findoptionS("urlregexplist"));
+			std::string header_regexp_list_location(findoptionS("headerregexplist"));
 			std::string exceptions_site_list_location(findoptionS("exceptionsitelist"));
 			std::string exceptions_url_list_location(findoptionS("exceptionurllist"));
 			std::string exception_extension_list_location(findoptionS("exceptionextensionlist"));
 			std::string exception_mimetype_list_location(findoptionS("exceptionmimetypelist"));
 			std::string exception_file_site_list_location(findoptionS("exceptionfilesitelist"));
+			std::string exception_file_url_list_location(findoptionS("exceptionfileurllist"));
+			std::string log_url_list_location(findoptionS("logurllist"));
+			std::string log_site_list_location(findoptionS("logsitelist"));
+			std::string log_regexpurl_list_location(findoptionS("logregexpurllist"));
 
-			if (enable_PICS == 1) {
+			if (enable_PICS) {
 				pics_rsac_nudity = findoptionI("RSACnudity");
 				pics_rsac_language = findoptionI("RSAClanguage");
 				pics_rsac_sex = findoptionI("RSACsex");
@@ -569,23 +597,23 @@ bool FOptionContainer::read(const char *filename)
 					return false;
 				}		// mime types
 				banned_mimetype_flag = true;
-			} else {
-#ifdef DGDEBUG
-				std::cout << "Blanket download block enabled; using exception file lists" << std::endl;
-#endif
-				if (!readFile(exception_extension_list_location.c_str(),&exception_extension_list,false,false,"exceptionextensionlist")) {
-					return false;
-				}		// file extensions
-				exception_extension_flag = true;
-				if (!readFile(exception_mimetype_list_location.c_str(),&exception_mimetype_list,false,true,"exceptionmimetypelist")) {
-					return false;
-				}		// mime types
-				exception_mimetype_flag = true;
 			}
+			if (!readFile(exception_extension_list_location.c_str(),&exception_extension_list,false,false,"exceptionextensionlist")) {
+				return false;
+			}		// file extensions
+			exception_extension_flag = true;
+			if (!readFile(exception_mimetype_list_location.c_str(),&exception_mimetype_list,false,true,"exceptionmimetypelist")) {
+				return false;
+			}		// mime types
+			exception_mimetype_flag = true;
 			if (!readFile(exception_file_site_list_location.c_str(),&exception_file_site_list,false,true,"exceptionfilesitelist")) {
 				return false;
 			}		// download site exceptions
 			exception_file_site_flag = true;
+			if (!readFile(exception_file_url_list_location.c_str(),&exception_file_url_list,true,true,"exceptionfileurllist")) {
+				return false;
+			}		// download site exceptions
+			exception_file_url_flag = true;
 
 			if (!readbplfile(banned_phrase_list_location.c_str(), exception_phrase_list_location.c_str(), weighted_phrase_list_location.c_str())) {
 				return false;
@@ -615,46 +643,67 @@ bool FOptionContainer::read(const char *filename)
 				return false;
 			}		// grey urls
 			grey_url_flag = true;
+			
+			// log-only lists
+			if (log_url_list_location.length() && readFile(log_url_list_location.c_str(), &log_url_list, true, true, "logurllist")) {
+				log_url_flag = true;
+#ifdef DGDEBUG
+				std::cout << "Enabled log-only URL list" << std::endl;
+#endif
+			}
+			if (log_site_list_location.length() && readFile(log_site_list_location.c_str(), &log_site_list, false, true, "logsitelist")) {
+				log_site_flag = true;
+#ifdef DGDEBUG
+				std::cout << "Enabled log-only domain list" << std::endl;
+#endif
+			}
+			if (log_regexpurl_list_location.length() && readRegExMatchFile(log_regexpurl_list_location.c_str(), "logregexpurllist", log_regexpurl_list,
+				log_regexpurl_list_comp, log_regexpurl_list_source, log_regexpurl_list_ref))
+			{
+				log_regexpurl_flag = true;
+#ifdef DGDEBUG
+				std::cout << "Enabled log-only RegExp URL list" << std::endl;
+#endif
+			}
 
-			if (!readRegExURLFile(banned_regexpurl_list_location.c_str(),"bannedregexpurllist",banned_regexpurl_list,
+			if (!readRegExMatchFile(banned_regexpurl_list_location.c_str(),"bannedregexpurllist",banned_regexpurl_list,
 				banned_regexpurl_list_comp, banned_regexpurl_list_source, banned_regexpurl_list_ref))
 			{
 				return false;
 			}		// banned reg exp urls
 			banned_regexpurl_flag = true;
 
-			if (!readRegExURLFile(exception_regexpurl_list_location.c_str(),"exceptionregexpurllist",exception_regexpurl_list,
+			if (!readRegExMatchFile(exception_regexpurl_list_location.c_str(),"exceptionregexpurllist",exception_regexpurl_list,
 				exception_regexpurl_list_comp, exception_regexpurl_list_source, exception_regexpurl_list_ref))
 			{
 				return false;
 			}		// exception reg exp urls
 			exception_regexpurl_flag = true;
 
-			if (!readRegExListFile(content_regexp_list_location.c_str(),"contentregexplist",content_regexp_list,content_regexp_list_rep,content_regexp_list_comp)) {
+			if (!readRegExMatchFile(banned_regexpheader_list_location.c_str(), "bannedregexpheaderlist", banned_regexpheader_list,
+				banned_regexpheader_list_comp, banned_regexpheader_list_source, banned_regexpheader_list_ref))
+			{
+				return false;
+			}		// banned reg exp headers
+			banned_regexpheader_flag = true;
+
+			if (!readRegExReplacementFile(content_regexp_list_location.c_str(),"contentregexplist",content_regexp_list,content_regexp_list_rep,content_regexp_list_comp)) {
 				return false;
 			}		// content replacement regular expressions
 			content_regexp_flag = true;
 
-			if (!readRegExListFile(url_regexp_list_location.c_str(),"urlregexplist",url_regexp_list,url_regexp_list_rep,url_regexp_list_comp)) {
+			if (!readRegExReplacementFile(url_regexp_list_location.c_str(),"urlregexplist",url_regexp_list,url_regexp_list_rep,url_regexp_list_comp)) {
 				return false;
 			}  // url replacement regular expressions
 			url_regexp_flag = true;
+
+			if (!readRegExReplacementFile(header_regexp_list_location.c_str(), "headerregexplist", header_regexp_list, header_regexp_list_rep, header_regexp_list_comp)) {
+				return false;
+			}  // header replacement regular expressions
+			header_regexp_flag = true;
 #ifdef DGDEBUG
 			std::cout << "Lists in memory" << std::endl;
 #endif
-
-			if (o.lm.l[banned_site_list]->inList("**")) {
-				blanketblock = true;
-			}
-			if (o.lm.l[banned_site_list]->inList("*ip")) {
-				blanket_ip_block = true;
-			}
-			if (o.lm.l[banned_site_list]->inList("**s")) {
-				blanketsslblock = true;
-			}
-			if (o.lm.l[banned_site_list]->inList("*ips")) {
-				blanketssl_ip_block = true;
-			}
 		}
 
 		if (!precompileregexps()) {
@@ -708,12 +757,12 @@ bool FOptionContainer::read(const char *filename)
 			std::cout << "Setting imagic key to '" << imagic << "'" << std::endl;
 #endif
 			if (findoptionS("infectionbypasserrorsonly") == "off") {
-				infection_bypass_errors_only = 0;
+				infection_bypass_errors_only = false;
 			} else {
 #ifdef DGDEBUG
 				std::cout << "Only allowing infection bypass on scan error" << std::endl;
 #endif
-				infection_bypass_errors_only = 1;
+				infection_bypass_errors_only = true;
 			}
 		}
 	}
@@ -773,7 +822,8 @@ bool FOptionContainer::readbplfile(const char *banned, const char *exception, co
 				return false;
 			}
 		}
-		(*o.lm.l[banned_phrase_list]).makeGraph(force_quick_search);
+		if (!(*o.lm.l[banned_phrase_list]).makeGraph(force_quick_search))
+			return false;
 
 		(*o.lm.l[banned_phrase_list]).used = true;
 	}
@@ -781,7 +831,7 @@ bool FOptionContainer::readbplfile(const char *banned, const char *exception, co
 }
 
 // read regexp url list
-bool FOptionContainer::readRegExURLFile(const char *filename, const char *listname, unsigned int& listref,
+bool FOptionContainer::readRegExMatchFile(const char *filename, const char *listname, unsigned int& listref,
 	std::deque<RegExp> &list_comp, std::deque<String> &list_source, std::deque<unsigned int> &list_ref)
 {
 	int result = o.lm.newItemList(filename, true, 32, true);
@@ -793,16 +843,16 @@ bool FOptionContainer::readRegExURLFile(const char *filename, const char *listna
 		return false;
 	}
 	listref = (unsigned) result;
-	return compileRegExURLFile(listref, list_comp, list_source, list_ref);
+	return compileRegExMatchFile(listref, list_comp, list_source, list_ref);
 }
 
 // NOTE TO SELF - MOVE TO LISTCONTAINER TO SOLVE FUDGE
 // compile regexp url list
-bool FOptionContainer::compileRegExURLFile(unsigned int list, std::deque<RegExp> &list_comp,
+bool FOptionContainer::compileRegExMatchFile(unsigned int list, std::deque<RegExp> &list_comp,
 	std::deque<String> &list_source, std::deque<unsigned int> &list_ref)
 {
 	for (unsigned int i = 0; i < (*o.lm.l[list]).morelists.size(); i++) {
-		if (!compileRegExURLFile((*o.lm.l[list]).morelists[i],list_comp,list_source,list_ref)) {
+		if (!compileRegExMatchFile((*o.lm.l[list]).morelists[i],list_comp,list_source,list_ref)) {
 			return false;
 		}
 	}
@@ -830,7 +880,7 @@ bool FOptionContainer::compileRegExURLFile(unsigned int list, std::deque<RegExp>
 }
 
 // content and URL regular expression replacement files
-bool FOptionContainer::readRegExListFile(const char *filename, const char *listname, unsigned int& listid,
+bool FOptionContainer::readRegExReplacementFile(const char *filename, const char *listname, unsigned int& listid,
 	std::deque<String> &list_rep, std::deque<RegExp> &list_comp)
 {
 	int result = o.lm.newItemList(filename, true, 32, true);
@@ -880,11 +930,41 @@ bool FOptionContainer::readRegExListFile(const char *filename, const char *listn
 	return true;
 }
 
+// Recursively check site & URL lists for blanket matches
+char *FOptionContainer::testBlanketBlock(unsigned int list, bool ip, bool ssl) {
+	if (not o.lm.l[list]->isNow())
+		return NULL;
+	if (o.lm.l[list]->blanketblock) {
+		return (char*)o.language_list.getTranslation(502);
+	} else if (o.lm.l[list]->blanket_ip_block and ip) {
+		return (char*)o.language_list.getTranslation(505);
+	} else if (o.lm.l[list]->blanketsslblock and ssl) {
+		return (char*)o.language_list.getTranslation(506);
+	} else if (o.lm.l[list]->blanketssl_ip_block and ssl and ip) {
+		return (char*)o.language_list.getTranslation(507);
+	}
+	for (std::vector<unsigned int>::iterator i = o.lm.l[list]->morelists.begin(); i != o.lm.l[list]->morelists.end(); i++) {
+		char *r = testBlanketBlock(*i, ip, ssl);
+		if (r) {
+			return r;
+		}
+	}
+	return NULL;
+}
+
 // checkme: there's an awful lot of removing whitespace, PTP, etc. going on here.
 // perhaps connectionhandler could keep a suitably modified version handy to prevent repitition of work?
 
-char *FOptionContainer::inSiteList(String &url, unsigned int list)
+char *FOptionContainer::inSiteList(String &url, unsigned int list, bool doblanket, bool ip, bool ssl)
 {
+	// Perform blanket matching if desired
+	if (doblanket) {
+		char *r = testBlanketBlock(list, ip, ssl);
+		if (r) {
+			return r;
+		}
+	}
+
 	url.removeWhiteSpace();  // just in case of weird browser crap
 	url.toLower();
 	url.removePTP();  // chop off the ht(f)tp(s)://
@@ -893,7 +973,7 @@ char *FOptionContainer::inSiteList(String &url, unsigned int list)
 	}
 	char *i;
 	bool isipurl = isIPHostname(url);
-	if (reverse_lookups == 1 && isipurl) {	// change that ip into hostname
+	if (reverse_lookups && isipurl) {	// change that ip into hostname
 		std::deque<String > *url2s = ipToHostname(url.toCharArray());
 		String url2;
 		for (std::deque<String>::iterator j = url2s->begin(); j != url2s->end(); j++) {
@@ -927,29 +1007,40 @@ char *FOptionContainer::inSiteList(String &url, unsigned int list)
 
 // checkme: remove things like this & make inSiteList/inIPList public?
 
-char *FOptionContainer::inBannedSiteList(String url)
+char *FOptionContainer::inBannedSiteList(String url, bool doblanket, bool ip, bool ssl)
 {
-	return inSiteList(url, banned_site_list);
+	return inSiteList(url, banned_site_list, doblanket, ip, ssl);
 }
 
-bool FOptionContainer::inGreySiteList(String url)
+bool FOptionContainer::inGreySiteList(String url, bool doblanket, bool ip, bool ssl)
 {
-	return inSiteList(url, grey_site_list) != NULL;
+	return inSiteList(url, grey_site_list, doblanket, ip, ssl) != NULL;
 }
 
-bool FOptionContainer::inExceptionSiteList(String url)
+bool FOptionContainer::inExceptionSiteList(String url, bool doblanket, bool ip, bool ssl)
 {
-	return inSiteList(url, exception_site_list) != NULL;
+	return inSiteList(url, exception_site_list, doblanket, ip, ssl) != NULL;
 }
 
 bool FOptionContainer::inExceptionFileSiteList(String url)
 {
-	return inSiteList(url, exception_file_site_list) != NULL;
+	if (inSiteList(url, exception_file_site_list) != NULL)
+		return true;
+	else
+		return inURLList(url, exception_file_url_list) != NULL;
 }
 
 // look in given URL list for given URL
-char *FOptionContainer::inURLList(String &url, unsigned int list) {
-	int fl;
+char *FOptionContainer::inURLList(String &url, unsigned int list, bool doblanket, bool ip, bool ssl) {
+	// Perform blanket matching if desired
+	if (doblanket) {
+		char *r = testBlanketBlock(list, ip, ssl);
+		if (r) {
+			return r;
+		}
+	}
+
+	unsigned int fl;
 	char *i;
 	String foundurl;
 #ifdef DGDEBUG
@@ -972,7 +1063,7 @@ char *FOptionContainer::inURLList(String &url, unsigned int list) {
 #ifdef DGDEBUG
 	std::cout << "inURLList (processed): " << url << std::endl;
 #endif
-	if (reverse_lookups == 1 && url.after("/").length() > 0) {
+	if (reverse_lookups && url.after("/").length() > 0) {
 		String hostname(url.before("/"));
 		if (isIPHostname(hostname)) {
 			std::deque<String > *url2s = ipToHostname(hostname.toCharArray());
@@ -1025,28 +1116,58 @@ char *FOptionContainer::inURLList(String &url, unsigned int list) {
 	return NULL;
 }
 
-char *FOptionContainer::inBannedURLList(String url)
+char *FOptionContainer::inBannedURLList(String url, bool doblanket, bool ip, bool ssl)
 {
 #ifdef DGDEBUG
 	std::cout<<"inBannedURLList"<<std::endl;
 #endif
-	return inURLList(url, banned_url_list);
+	return inURLList(url, banned_url_list, doblanket, ip, ssl);
 }
 
-bool FOptionContainer::inGreyURLList(String url)
+bool FOptionContainer::inGreyURLList(String url, bool doblanket, bool ip, bool ssl)
 {
 #ifdef DGDEBUG
 	std::cout<<"inGreyURLList"<<std::endl;
 #endif
-	return inURLList(url, grey_url_list) != NULL;
+	return inURLList(url, grey_url_list, doblanket, ip, ssl) != NULL;
 }
 
-bool FOptionContainer::inExceptionURLList(String url)
+bool FOptionContainer::inExceptionURLList(String url, bool doblanket, bool ip, bool ssl)
 {
 #ifdef DGDEBUG
 	std::cout<<"inExceptionURLList"<<std::endl;
 #endif
-	return inURLList(url, exception_url_list) != NULL;
+	return inURLList(url, exception_url_list, doblanket, ip, ssl) != NULL;
+}
+
+// New log-only site lists
+const char* FOptionContainer::inLogURLList(String url)
+{
+	if (!log_url_flag)
+		return NULL;
+	if (inURLList(url, log_url_list) != NULL) {
+		return o.lm.l[log_url_list]->lastcategory.toCharArray();
+	}
+	return NULL;
+}
+
+const char* FOptionContainer::inLogSiteList(String url)
+{
+	if (!log_site_flag)
+		return NULL;
+	if (inSiteList(url, log_site_list) != NULL) {
+		return o.lm.l[log_site_list]->lastcategory.toCharArray();
+	}
+	return NULL;
+}
+
+const char* FOptionContainer::inLogRegExpURLList(String url) {
+	if (!log_regexpurl_flag)
+		return NULL;
+	int j = inRegExpURLList(url, log_regexpurl_list_comp, log_regexpurl_list_ref, log_regexpurl_list);
+	if (j == -1)
+		return NULL;
+	return o.lm.l[log_regexpurl_list_ref[j]]->category.toCharArray();
 }
 
 // TODO: Store the modified URL somewhere, instead of re-processing it every time.
@@ -1064,6 +1185,31 @@ char *FOptionContainer::inExtensionList(unsigned int list, String url)
 	return (*o.lm.l[list]).findEndsWith(url.toCharArray());
 }
 
+// is this line of the headers in the banned regexp header list?
+int FOptionContainer::inBannedRegExpHeaderList(std::deque<String> &header)
+{
+
+	for (std::deque<String>::iterator k = header.begin(); k != header.end(); k++) {
+#ifdef DGDEBUG
+		std::cout << "inBannedRegExpHeaderList: " << *k << std::endl;
+#endif
+		unsigned int i = 0;
+		for (std::deque<RegExp>::iterator j = banned_regexpheader_list_comp.begin(); j != banned_regexpheader_list_comp.end(); j++) {
+			if (o.lm.l[banned_regexpheader_list_ref[i]]->isNow()) {
+				j->match(k->toCharArray());
+				if (j->matched())
+					return i;
+			}
+#ifdef DGDEBUG
+			else
+				std::cout << "Outside included regexp list's time limit" << std::endl;
+#endif
+			i++;
+		}
+	}
+	return -1;
+}
+
 // is this URL in the given regexp URL list?
 int FOptionContainer::inRegExpURLList(String &url, std::deque<RegExp> &list_comp, std::deque<unsigned int> &list_ref, unsigned int list)
 {
@@ -1105,15 +1251,15 @@ int FOptionContainer::inRegExpURLList(String &url, std::deque<RegExp> &list_comp
 #endif
 		unsigned int i = 0;
 		for (std::deque<RegExp>::iterator j = list_comp.begin(); j != list_comp.end(); j++) {
-			j->match(url.toCharArray());
-			if (j->matched()) {
-				if (o.lm.l[list_ref[i]]->isNow())
+			if (o.lm.l[list_ref[i]]->isNow()) {
+				j->match(url.toCharArray());
+				if (j->matched())
 					return i;
-#ifdef DGDEBUG
-				else
-					std::cout << "Outside included regexp list's time limit" << std::endl;
-#endif
 			}
+#ifdef DGDEBUG
+			else
+				std::cout << "Outside included regexp list's time limit" << std::endl;
+#endif
 			i++;
 		}
 	}
