@@ -33,6 +33,7 @@
 
 #include <syslog.h>
 #include <csignal>
+#include <ctime>
 #include <sys/stat.h>
 #include <pwd.h>
 #include <cerrno>
@@ -105,8 +106,8 @@ extern "C"
 }
 
 // logging & URL cache processes
-int log_listener(std::string log_location, int logconerror, int logsyslog);
-int url_list_listener(int logconerror);
+int log_listener(std::string log_location, bool logconerror, bool logsyslog);
+int url_list_listener(bool logconerror);
 // send flush message over URL cache IPC socket
 void flush_urlcache();
 
@@ -315,7 +316,7 @@ void flush_urlcache()
 bool daemonise()
 {
 
-	if (o.no_daemon == 1) {
+	if (o.no_daemon) {
 		return true;
 	}
 #ifdef DGDEBUG
@@ -523,7 +524,7 @@ int handle_connections(UDSocket &pipe)
 
 		// now check the connection is actually good
 		if (peersock->getFD() < 0 || peersockip.length() < 7) {
-			if (o.logconerror == 1)
+			if (o.logconerror)
 				syslog(LOG_INFO, "Error accepting. (Ignorable)");
 			continue;
 		}
@@ -581,7 +582,7 @@ bool getsock_fromparent(UDSocket &fd/*, int &socknum*/)
 		// so effectively providing a lock
 	}
 	catch(exception & e) {
-		if (o.logconerror == 1)
+		if (o.logconerror)
 			syslog(LOG_ERR, "Error telling parent we accepted: %s", e.what());
 		peersock->close();
 		return false;
@@ -843,7 +844,7 @@ void tellchild_accept(int num, int whichsock)
 // *
 
 
-int log_listener(std::string log_location, int logconerror, int logsyslog)
+int log_listener(std::string log_location, bool logconerror, bool logsyslog)
 {
 #ifdef DGDEBUG
 	std::cout << "log listener started" << std::endl;
@@ -854,31 +855,26 @@ int log_listener(std::string log_location, int logconerror, int logsyslog)
 	UDSocket* ipcpeersock;  // the socket which will contain the ipc connection
 	int rc, ipcsockfd;
 
-	char *logline = new char[8192];
-
 #ifdef __EMAIL
 	// Email notification patch by J. Gauthier
-	char *vbody_temp = new char[8192];   
-   
 	map<string,int> violation_map;
 	map<string,int> timestamp_map;   
 	map<string,string> vbody_map;
 
 	int curv_tmp, stamp_tmp, byuser;
-	int smtppid;
 #endif
 	
 	//String where, what, how;
 	std::string cr("\n");
    
-	std::string where, what, how, cat, clienthost, from, who, mimetype;
-	int port = 80, size = 0, isnaughty = 0, isexception = 0, code = 200;
-	int cachehit = 0, wasinfected = 0, wasscanned = 0, naughtiness = 0, filtergroup = 0;
+	std::string where, what, how, cat, clienthost, from, who, mimetype, useragent, ssize, sweight;
+	int port = 80, isnaughty = 0, isexception = 0, code = 200;
+	int cachehit = 0, wasinfected = 0, wasscanned = 0, filtergroup = 0;
 	long tv_sec = 0, tv_usec = 0;
-	int contentmodified = 0, urlmodified = 0;
+	int contentmodified = 0, urlmodified = 0, headermodified = 0;
 
 	ofstream* logfile = NULL;
-	if (logsyslog == 0) {
+	if (!logsyslog) {
 		logfile = new ofstream(log_location.c_str(), ios::app);
 		if (logfile->fail()) {
 			syslog(LOG_ERR, "Error opening/creating log file.");
@@ -905,13 +901,13 @@ int log_listener(std::string log_location, int logconerror, int logsyslog)
 			if (errno == EINTR) {
 				continue;  // was interupted by a signal so restart
 			}
-			if (logconerror == 1) {
+			if (logconerror) {
 				syslog(LOG_ERR, "ipc rc<0. (Ignorable)");
 			}
 			continue;
 		}
 		if (rc < 1) {
-			if (logconerror == 1) {
+			if (logconerror) {
 				syslog(LOG_ERR, "ipc rc<1. (Ignorable)");
 			}
 			continue;
@@ -923,7 +919,7 @@ int log_listener(std::string log_location, int logconerror, int logsyslog)
 			ipcpeersock = loggersock.accept();
 			if (ipcpeersock->getFD() < 0) {
 				delete ipcpeersock;
-				if (logconerror == 1) {
+				if (logconerror) {
 					syslog(LOG_ERR, "Error accepting ipc. (Ignorable)");
 				}
 				continue;  // if the fd of the new socket < 0 there was error
@@ -937,8 +933,9 @@ int log_listener(std::string log_location, int logconerror, int logsyslog)
 			// read in the various parts of the log string
 			bool error = false;
 			int itemcount = 0;
-			while(itemcount < 23) {
+			while(itemcount < (o.log_user_agent ? 24 : 23)) {
 				try {
+				    	char *logline = new char[8192];
 					rc = ipcpeersock->getLine(logline, 8192, 3, true);  // throws on err
 					if (rc < 0) {
 						delete ipcpeersock;
@@ -960,7 +957,7 @@ int log_listener(std::string log_location, int logconerror, int logsyslog)
 							isnaughty = atoi(logline);
 							break;
 						case 3:
-							naughtiness = atoi(logline);
+							sweight = logline;
 							break;
 						case 4:
 							where = logline;
@@ -993,35 +990,42 @@ int log_listener(std::string log_location, int logconerror, int logsyslog)
 							urlmodified = atoi(logline);
 							break;
 						case 14:
-							size = atoi(logline);
+							headermodified = atoi(logline);
 							break;
 						case 15:
-							filtergroup = atoi(logline);
+							ssize = logline;
 							break;
 						case 16:
-							code = atoi(logline);
+							filtergroup = atoi(logline);
 							break;
 						case 17:
-							cachehit = atoi(logline);
+							code = atoi(logline);
 							break;
 						case 18:
-							mimetype = logline;
+							cachehit = atoi(logline);
 							break;
 						case 19:
-							tv_sec = atol(logline);
+							mimetype = logline;
 							break;
 						case 20:
-							tv_usec = atol(logline);
+							tv_sec = atol(logline);
 							break;
 						case 21:
+							tv_usec = atol(logline);
+							break;
+						case 22:
 							clienthost = logline;
 							break;
+						case 23:
+							useragent = logline;
+							break;
 						}
 					}
+					delete[]logline;
 				}
 				catch(exception & e) {
 					delete ipcpeersock;
-					if (logconerror == 1)
+					if (logconerror)
 						syslog(LOG_ERR, "Error reading ipc. (Ignorable)");
 					error = true;
 					break;
@@ -1037,7 +1041,7 @@ int log_listener(std::string log_location, int logconerror, int logsyslog)
 
 			if (port != 0 && port != 80) {
 				// put port numbers of non-standard HTTP requests into the logged URL
-				String newwhere(where.c_str());
+				String newwhere(where);
 				if (newwhere.after("://").contains("/")) {
 					String proto, host, path;
 					proto = newwhere.before("://");
@@ -1051,10 +1055,10 @@ int log_listener(std::string log_location, int logconerror, int logsyslog)
 					newwhere += String((int) port);
 					newwhere += "/";
 					newwhere += path;
-					where = newwhere.toCharArray();
+					where = newwhere;
 				} else {
 					where += ":";
-					where += String((int) port).toCharArray();
+					where += String((int) port);
 				}
 			}
 
@@ -1062,10 +1066,8 @@ int log_listener(std::string log_location, int logconerror, int logsyslog)
 			if (isnaughty) {
 				what = "*DENIED* " + what;
 			}
-			else if (isexception) {
-				if (o.log_exception_hits == 1) {
-					what = "*EXCEPTION* " + what;
-				}
+			else if (isexception && (o.log_exception_hits == 2)) {
+				what = "*EXCEPTION* " + what;
 			}
 		   
 			if (wasscanned) {
@@ -1081,12 +1083,15 @@ int log_listener(std::string log_location, int logconerror, int logsyslog)
 			if (urlmodified) {
 				what = "*URLMOD* " + what;
 			}
+			if (headermodified) {
+				what = "*HEADERMOD* " + what;
+			}
 
-			std::string builtline, year, month, day, hour, min, sec, when, ssize, sweight, vbody, utime;
+			std::string builtline, year, month, day, hour, min, sec, when, vbody, utime;
 			struct timeval theend;
 
 			// create a string representation of UNIX timestamp if desired
-			if ((o.log_timestamp == 1) || (o.log_file_format == 3)) {
+			if (o.log_timestamp || (o.log_file_format == 3)) {
 				gettimeofday(&theend, NULL);
 				String temp((int) (theend.tv_usec / 1000));
 				while (temp.length() < 3) {
@@ -1095,9 +1100,9 @@ int log_listener(std::string log_location, int logconerror, int logsyslog)
 				if (temp.length() > 3) {
 					temp = "999";
 				}
-				utime = temp.toCharArray();
+				utime = temp;
 				utime = "." + utime;
-				utime = String((int) theend.tv_sec).toCharArray() + utime;
+				utime = String((int) theend.tv_sec) + utime;
 			}
 
 			if (o.log_file_format != 3) {
@@ -1107,33 +1112,30 @@ int log_listener(std::string log_location, int logconerror, int logsyslog)
 				struct tm *tmnow;  // to hold the result from localtime()
 				time(&tnow);  // get the time after the lock so all entries in order
 				tmnow = localtime(&tnow);  // convert to local time (BST, etc)
-				year = String(tmnow->tm_year + 1900).toCharArray();
-				month = String(tmnow->tm_mon + 1).toCharArray();
-				day = String(tmnow->tm_mday).toCharArray();
-				hour = String(tmnow->tm_hour).toCharArray();
+				year = String(tmnow->tm_year + 1900);
+				month = String(tmnow->tm_mon + 1);
+				day = String(tmnow->tm_mday);
+				hour = String(tmnow->tm_hour);
 				temp = String(tmnow->tm_min);
 				if (temp.length() == 1) {
 					temp = "0" + temp;
 				}
-				min = temp.toCharArray();
+				min = temp;
 				temp = String(tmnow->tm_sec);
 				if (temp.length() == 1) {
 					temp = "0" + temp;
 				}
-				sec = temp.toCharArray();
+				sec = temp;
 				when = year + "." + month + "." + day + " " + hour + ":" + min + ":" + sec;
 				// truncate long log items
 				/*if ((o.max_logitem_length > 0) && (when.length() > o.max_logitem_length))
 					when.resize(o.max_logitem_length);*/
 				// append timestamp if desired
-				if (o.log_timestamp == 1)
+				if (o.log_timestamp)
 					when += " " + utime;
 					
 			}
 
-			sweight = String(naughtiness).toCharArray();
-			ssize = String(size).toCharArray();
-
 			// truncate long log items
 			if (o.max_logitem_length > 0) {
 				//where.limitLength(o.max_logitem_length);
@@ -1154,21 +1156,21 @@ int log_listener(std::string log_location, int logconerror, int logsyslog)
 			}
 
 			// blank out IP, hostname and username if desired
-			if (o.anonymise_logs == 1) {
+			if (o.anonymise_logs) {
 				who = "";
 				from = "0.0.0.0";
-				//delete clienthost;
-				clienthost[0]=0;
+				clienthost.clear();
 			}
 
-			std::string stringcode(String(code).toCharArray());
-			std::string stringgroup(String(filtergroup+1).toCharArray());
+			String stringcode(code);
+			String stringgroup(filtergroup+1);
 
 			switch (o.log_file_format) {
 			case 4:
 				builtline = when +"\t"+ who + "\t" + from + "\t" + where + "\t" + what + "\t" + how
 					+ "\t" + ssize + "\t" + sweight + "\t" + cat +  "\t" + stringgroup + "\t"
-					+ stringcode + "\t" + mimetype + "\t" + clienthost + "\t" + o.fg[filtergroup]->name;
+					+ stringcode + "\t" + mimetype + "\t" + clienthost + "\t" + o.fg[filtergroup]->name
+					+ "\t" + (o.log_user_agent ? useragent : "-");
 				break;
 			case 3:
 				{
@@ -1182,17 +1184,17 @@ int log_listener(std::string log_location, int logconerror, int logsyslog)
 					while (temp.length() < 6) {
 						temp = " " + temp;
 					}
-					duration = temp.toCharArray();
+					duration = temp;
 
 					if (code == 403) {
 						hitmiss = "TCP_DENIED/403";
 					} else {
 						if (cachehit) {
 							hitmiss = "TCP_HIT/";
-							hitmiss += stringcode;
+							hitmiss.append(stringcode);
 						} else {
 							hitmiss = "TCP_MISS/";
-							hitmiss += stringcode;
+							hitmiss.append(stringcode);
 						}
 					}
 					hier = "DEFAULT_PARENT/";
@@ -1216,15 +1218,17 @@ int log_listener(std::string log_location, int logconerror, int logsyslog)
 			case 2:
 				builtline = "\"" + when  +"\",\""+ who + "\",\"" + from + "\",\"" + where + "\",\"" + what + "\",\""
 					+ how + "\",\"" + ssize + "\",\"" + sweight + "\",\"" + cat +  "\",\"" + stringgroup + "\",\""
-					+ stringcode + "\",\"" + mimetype + "\",\"" + clienthost + "\",\"" + o.fg[filtergroup]->name + "\"";
+					+ stringcode + "\",\"" + mimetype + "\",\"" + clienthost + "\",\"" + o.fg[filtergroup]->name + "\",\""
+					+ (o.log_user_agent ? useragent : "-") + "\"";
 				break;
 			default:
 				builtline = when +" "+ who + " " + from + " " + where + " " + what + " "
 					+ how + " " + ssize + " " + sweight + " " + cat +  " " + stringgroup + " "
-					+ stringcode + " " + mimetype + " " + clienthost + " " + o.fg[filtergroup]->name;
+					+ stringcode + " " + mimetype + " " + clienthost + " " + o.fg[filtergroup]->name + " "
+					+ (o.log_user_agent ? useragent : "-");
 			}
 
-			if (logsyslog == 0)
+			if (!logsyslog)
 				*logfile << builtline << std::endl;  // append the line
 			else
 				syslog(LOG_INFO, "%s", builtline.c_str());
@@ -1242,37 +1246,49 @@ int log_listener(std::string log_location, int logconerror, int logsyslog)
 
 				// virus
 				if ((wasscanned && wasinfected) && (o.fg[filtergroup]->notifyav))  {
-					if ((smtppid = fork() == 0))  {
-						FILE* mail = popen (o.mailer.c_str(), "w");
-						if (mail==NULL) {
-							syslog(LOG_ERR, "Unable to contact defined mailer.");
-						}
-						else {
-							fprintf(mail, "To: %s\n", o.fg[filtergroup]->avadmin.c_str());
-							fprintf(mail, "From: %s\n", o.fg[filtergroup]->mailfrom.c_str());
-							fprintf(mail, "Subject: %s\n", o.fg[filtergroup]->avsubject.c_str());
-							fprintf(mail, "A virus was detected by DansGuardian.\n\n");
-							fprintf(mail, "%-10s%s\n", "Data/Time:", when.c_str());
-							if (who != "-")
-								fprintf(mail, "%-10s%s\n", "User:", who.c_str());
-							fprintf(mail, "%-10s%s (%s)\n", "From:", from.c_str(),  ((clienthost.length() > 0) ? clienthost.c_str() : "-"));
-							fprintf(mail, "%-10s%s\n", "Where:", where.c_str());
-							// specifically, the virus name comes after message 1100 ("Virus or bad content detected.")
-							String swhat(what);
-							fprintf(mail, "%-10s%s\n", "Why:", swhat.after(o.language_list.getTranslation(1100)).toCharArray() + 1);
-							fprintf(mail, "%-10s%s\n", "Method:", how.c_str());
-							fprintf(mail, "%-10s%s\n", "Size:", ssize.c_str());
-							fprintf(mail, "%-10s%s\n", "Weight:", sweight.c_str());
-							if (cat.c_str()!=NULL)
-								fprintf(mail, "%-10s%s\n", "Category:", cat.c_str());
-							fprintf(mail, "%-10s%s\n", "Mime type:", mimetype.c_str());
-							fprintf(mail, "%-10s%s\n", "Group:", o.fg[filtergroup]->name.c_str());
-							fprintf(mail, "%-10s%s\n", "HTTP resp:", stringcode.c_str());
+					// Use a double fork to ensure child processes are reaped adequately.
+					pid_t smtppid;
+					if ((smtppid = fork()) != 0) {
+						// Parent immediately waits for first child
+						waitpid(smtppid, NULL, 0);
+					} else {
+						// First child forks off the *real* process, but immediately exits itself
+						if (fork() == 0)  {
+							// Second child - do stuff
+							setsid();
+							FILE* mail = popen (o.mailer.c_str(), "w");
+							if (mail==NULL) {
+								syslog(LOG_ERR, "Unable to contact defined mailer.");
+							}
+							else {
+								fprintf(mail, "To: %s\n", o.fg[filtergroup]->avadmin.c_str());
+								fprintf(mail, "From: %s\n", o.fg[filtergroup]->mailfrom.c_str());
+								fprintf(mail, "Subject: %s\n", o.fg[filtergroup]->avsubject.c_str());
+								fprintf(mail, "A virus was detected by DansGuardian.\n\n");
+								fprintf(mail, "%-10s%s\n", "Data/Time:", when.c_str());
+								if (who != "-")
+									fprintf(mail, "%-10s%s\n", "User:", who.c_str());
+								fprintf(mail, "%-10s%s (%s)\n", "From:", from.c_str(),  ((clienthost.length() > 0) ? clienthost.c_str() : "-"));
+								fprintf(mail, "%-10s%s\n", "Where:", where.c_str());
+								// specifically, the virus name comes after message 1100 ("Virus or bad content detected.")
+								String swhat(what);
+								fprintf(mail, "%-10s%s\n", "Why:", swhat.after(o.language_list.getTranslation(1100)).toCharArray() + 1);
+								fprintf(mail, "%-10s%s\n", "Method:", how.c_str());
+								fprintf(mail, "%-10s%s\n", "Size:", ssize.c_str());
+								fprintf(mail, "%-10s%s\n", "Weight:", sweight.c_str());
+								if (cat.c_str()!=NULL)
+									fprintf(mail, "%-10s%s\n", "Category:", cat.c_str());
+								fprintf(mail, "%-10s%s\n", "Mime type:", mimetype.c_str());
+								fprintf(mail, "%-10s%s\n", "Group:", o.fg[filtergroup]->name.c_str());
+								fprintf(mail, "%-10s%s\n", "HTTP resp:", stringcode.c_str());
 
-							pclose(mail);
+								pclose(mail);
+							}
+							// Second child exits
+							_exit(0);
 						}
-						// child exits
-						_exit(0);				 
+						// First child exits
+						_exit(0);
 					}
 				}
 
@@ -1302,6 +1318,7 @@ int log_listener(std::string log_location, int logconerror, int logsyslog)
 						o.fg[filtergroup]->current_violations++;
 
 					// construct email report
+					char *vbody_temp = new char[8192];   
 					sprintf(vbody_temp, "%-10s%s\n", "Data/Time:", when.c_str());
 					vbody+=vbody_temp;
 
@@ -1331,6 +1348,7 @@ int log_listener(std::string log_location, int logconerror, int logsyslog)
 					vbody+=vbody_temp;
 					sprintf(vbody_temp, "%-10s%s\n\n", "HTTP resp:", stringcode.c_str());
 					vbody+=vbody_temp;
+					delete[] vbody_temp;
 					
 					// store the report with the group/user
 					if (byuser) {
@@ -1347,34 +1365,46 @@ int log_listener(std::string log_location, int logconerror, int logsyslog)
 					// if threshold exceeded, send mail
 					if (curv_tmp >= o.fg[filtergroup]->violations) {
 						if ((o.fg[filtergroup]->threshold == 0) || ( (time(0) - stamp_tmp) <= o.fg[filtergroup]->threshold)) {
-							if ((smtppid = fork() == 0)) {
-								FILE* mail = popen (o.mailer.c_str(), "w");
-								if (mail==NULL) {
-									syslog(LOG_ERR, "Unable to contact defined mailer.");
-								}
-								else {
-									fprintf(mail, "To: %s\n", o.fg[filtergroup]->contentadmin.c_str());
-									fprintf(mail, "From: %s\n", o.fg[filtergroup]->mailfrom.c_str());
+							// Use a double fork to ensure child processes are reaped adequately.
+							pid_t smtppid;
+							if ((smtppid = fork()) != 0) {
+								// Parent immediately waits for first child
+								waitpid(smtppid, NULL, 0);
+							} else {
+								// First child forks off the *real* process, but immediately exits itself
+								if (fork() == 0)  {
+									// Second child - do stuff
+									setsid();
+									FILE* mail = popen (o.mailer.c_str(), "w");
+									if (mail==NULL) {
+										syslog(LOG_ERR, "Unable to contact defined mailer.");
+									}
+									else {
+										fprintf(mail, "To: %s\n", o.fg[filtergroup]->contentadmin.c_str());
+										fprintf(mail, "From: %s\n", o.fg[filtergroup]->mailfrom.c_str());
 
-									if (byuser)
-										fprintf(mail, "Subject: %s (%s)\n", o.fg[filtergroup]->contentsubject.c_str(), who.c_str());
-									else
-										fprintf(mail, "Subject: %s\n", o.fg[filtergroup]->contentsubject.c_str());
+										if (byuser)
+											fprintf(mail, "Subject: %s (%s)\n", o.fg[filtergroup]->contentsubject.c_str(), who.c_str());
+										else
+											fprintf(mail, "Subject: %s\n", o.fg[filtergroup]->contentsubject.c_str());
 
-									fprintf(mail, "%i violation%s ha%s occured within %i seconds.\n",
-										curv_tmp,
-										(curv_tmp==1)?"":"s",
-										(curv_tmp==1)?"s":"ve",									 
-										o.fg[filtergroup]->threshold);
+										fprintf(mail, "%i violation%s ha%s occured within %i seconds.\n",
+											curv_tmp,
+											(curv_tmp==1)?"":"s",
+											(curv_tmp==1)?"s":"ve",									 
+											o.fg[filtergroup]->threshold);
 
-									fprintf(mail, "%s\n\n", "This exceeds the notification threshold.");
-									if (byuser)
-										fprintf(mail, "%s", vbody_map[who].c_str());
-									else
-										fprintf(mail, "%s", o.fg[filtergroup]->violationbody.c_str());
+										fprintf(mail, "%s\n\n", "This exceeds the notification threshold.");
+										if (byuser)
+											fprintf(mail, "%s", vbody_map[who].c_str());
+										else
+											fprintf(mail, "%s", o.fg[filtergroup]->violationbody.c_str());
 										pclose(mail);
 									}
-								// child exits
+									// Second child exits
+									_exit(0);
+								}
+								// First child exits
 								_exit(0);
 							}
 						}
@@ -1401,7 +1431,7 @@ int log_listener(std::string log_location, int logconerror, int logsyslog)
 	return 1;  // It is only possible to reach here with an error
 }
 
-int url_list_listener(int logconerror)
+int url_list_listener(bool logconerror)
 {
 #ifdef DGDEBUG
 	std::cout << "url listener started" << std::endl;
@@ -1444,7 +1474,7 @@ int url_list_listener(int logconerror)
 			if (errno == EINTR) {
 				continue;  // was interupted by a signal so restart
 			}
-			if (logconerror == 1) {
+			if (logconerror) {
 				syslog(LOG_ERR, "%s", "url ipc rc<0. (Ignorable)");
 			}
 			continue;
@@ -1456,7 +1486,7 @@ int url_list_listener(int logconerror)
 			ipcpeersock = urllistsock.accept();
 			if (ipcpeersock->getFD() < 0) {
 				delete ipcpeersock;
-				if (logconerror == 1) {
+				if (logconerror) {
 #ifdef DGDEBUG
 					std::cout << "Error accepting url ipc. (Ignorable)" << std::endl;
 #endif
@@ -1470,7 +1500,7 @@ int url_list_listener(int logconerror)
 			}
 			catch(exception & e) {
 				delete ipcpeersock;  // close the connection
-				if (logconerror == 1) {
+				if (logconerror) {
 #ifdef DGDEBUG
 					std::cout << "Error reading url ipc. (Ignorable)" << std::endl;
 					std::cerr << e.what() << std::endl;
@@ -1511,7 +1541,7 @@ int url_list_listener(int logconerror)
 			}
 			catch(exception & e) {
 				delete ipcpeersock;  // close the connection
-				if (logconerror == 1) {
+				if (logconerror) {
 					syslog(LOG_ERR, "%s", "Error writing url ipc. (Ignorable)");
 					syslog(LOG_ERR, "%s", e.what());
 				}
@@ -1529,7 +1559,7 @@ int url_list_listener(int logconerror)
 	return 1;  // It is only possible to reach here with an error
 }
 
-int ip_list_listener(std::string stat_location, int logconerror) {
+int ip_list_listener(std::string stat_location, bool logconerror) {
 #ifdef DGDEBUG
 	std::cout << "ip listener started" << std::endl;
 #endif
@@ -1582,7 +1612,7 @@ int ip_list_listener(std::string stat_location, int logconerror) {
 			if (errno == EINTR) {
 				continue;  // was interupted by a signal so restart
 			}
-			if (logconerror == 1) {
+			if (logconerror) {
 				syslog(LOG_ERR, "ip ipc rc<0. (Ignorable)");
 			}
 			continue;
@@ -1623,7 +1653,7 @@ int ip_list_listener(std::string stat_location, int logconerror) {
 			ipcpeersock = iplistsock.accept();
 			if (ipcpeersock->getFD() < 0) {
 				delete ipcpeersock;
-				if (logconerror == 1) {
+				if (logconerror) {
 #ifdef DGDEBUG
 					std::cout << "Error accepting ip ipc. (Ignorable)" << std::endl;
 #endif
@@ -1636,7 +1666,7 @@ int ip_list_listener(std::string stat_location, int logconerror) {
 				rc = ipcpeersock->getLine(inbuff, 16, 3);  // throws on err
 			} catch (exception& e) {
 				delete ipcpeersock;
-				if (logconerror == 1) {
+				if (logconerror) {
 #ifdef DGDEBUG
 					std::cout << "Error reading ip ipc. (Ignorable)" << std::endl;
 #endif
@@ -1658,7 +1688,7 @@ int ip_list_listener(std::string stat_location, int logconerror) {
 				ipcpeersock->writeToSockete(&reply, 1, 0, 6);
 			} catch (exception& e) {
 				delete ipcpeersock;
-				if (logconerror == 1) {
+				if (logconerror) {
 #ifdef DGDEBUG
 					std::cout << "Error writing ip ipc. (Ignorable)" << std::endl;
 #endif
@@ -1712,10 +1742,10 @@ int fc_controlit()
 	}
 
 
-	if (o.no_logger == 0) {
-		loggersock.reset();
-	} else {
+	if (o.no_logger) {
 		loggersock.close();
+	} else {
+		loggersock.reset();
 	}
 	if (o.url_cache_number > 0) {
 		urllistsock.reset();
@@ -1732,7 +1762,7 @@ int fc_controlit()
 	pid_t urllistpid = 0;  // url cache process id
 	pid_t iplistpid = 0; // ip cache process id
 
-	if (o.no_logger == 0) {
+	if (!o.no_logger) {
 		if (loggersock.getFD() < 0) {
 			if (!is_daemonised) {
 				std::cerr << "Error creating ipc socket" << std::endl;
@@ -1818,7 +1848,7 @@ int fc_controlit()
 	unlink(o.urlipc_filename.c_str());
 	unlink(o.ipipc_filename.c_str());
 
-	if (o.no_logger == 0) {
+	if (!o.no_logger) {
 		if (loggersock.bind((char *) o.ipc_filename.c_str())) {	// bind to file
 			if (!is_daemonised) {
 				std::cerr << "Error binding ipc server file (try using the SysV to stop DansGuardian then try starting it again or doing an 'rm " << o.ipc_filename << "')." << std::endl;
@@ -1855,7 +1885,7 @@ int fc_controlit()
 	}
 
 	if (o.max_ips > 0) {
-		if (iplistsock.bind((char*) o.ipipc_filename.c_str())) {	// bind to file
+		if (iplistsock.bind(o.ipipc_filename.c_str())) {	// bind to file
 			if (!is_daemonised) {
 				std::cerr << "Error binding iplistsock server file (try using the SysV to stop DansGuardian then try starting it again or doing an 'rm " << o.ipipc_filename << "')." << std::endl;
 			}
@@ -1918,7 +1948,7 @@ int fc_controlit()
 	// handle incoming TCP connections from the clients and one to handle
 	// incoming UDS ipc from our forked children.  This helps reduce
 	// bottlenecks by not having only one select() loop.
-	if (o.no_logger == 0) {
+	if (!o.no_logger) {
 
 		loggerpid = fork();  // make a child processes copy of self to be logger
 
@@ -1940,7 +1970,7 @@ int fc_controlit()
 		if (urllistpid == 0) {	// ma ma!  i am the child
 			serversockets.deleteAll(); // we don't need our copy of this so close it
 			delete[] serversockfds;
-			if (o.no_logger == 0) {
+			if (!o.no_logger) {
 				loggersock.close();  // we don't need our copy of this so close it
 			}
 			url_list_listener(o.logconerror);
@@ -1957,7 +1987,7 @@ int fc_controlit()
 		if (iplistpid == 0) {	// ma ma!  i am the child
 			serversockets.deleteAll(); // we don't need our copy of this so close it
 			delete[] serversockfds;
-			if (o.no_logger == 0) {
+			if (!o.no_logger) {
 				loggersock.close();  // we don't need our copy of this so close it
 			}
 			ip_list_listener(o.stat_location, o.logconerror);
@@ -1977,7 +2007,7 @@ int fc_controlit()
 	if (o.url_cache_number > 0) {
 		urllistsock.close();  // we don't need our copy of this so close it
 	}
-	if (o.no_logger == 0) {
+	if (!o.no_logger) {
 		loggersock.close();  // we don't need our copy of this so close it
 	}
 	if (o.max_ips > 0) {
@@ -1985,7 +2015,7 @@ int fc_controlit()
 	}
 
 	memset(&sa, 0, sizeof(sa));
-	if (o.soft_restart == 0) {
+	if (!o.soft_restart) {
 		sa.sa_handler = &sig_term;  // register sig_term as our handler
 	} else {
 		sa.sa_handler = &sig_termsafe;
@@ -2114,7 +2144,9 @@ int fc_controlit()
 				reloadconfig = true;  // filter groups problem so lets
 				// try and reload entire config instead
 				// if that fails it will bomb out
+				o.lm.garbageCollect();
 			} else {
+				o.lm.garbageCollect();
 				if (o.use_filter_groups_list) {
 					o.filter_groups_list.reset();
 					if (!o.doReadItemList(o.filter_groups_list_location.c_str(),&(o.filter_groups_list),"filtergroupslist",true))
@@ -2130,7 +2162,6 @@ int fc_controlit()
 							reloadconfig = true;  // auth plugs problem
 					}
 					if (!reloadconfig) {
-						o.lm.garbageCollect();
 						hup_allchildren();
 						prefork(o.min_children);
 						gentlereload = false;
@@ -2343,7 +2374,7 @@ int fc_controlit()
 	}
 
 	if (reloadconfig || ttg) {
-		if (o.no_logger == 0)
+		if (!o.no_logger)
 			::kill(loggerpid, SIGTERM);  // get rid of logger
 		if (o.url_cache_number > 0)
 			::kill(urllistpid, SIGTERM);  // get rid of url cache
@@ -2351,7 +2382,7 @@ int fc_controlit()
 			::kill(iplistpid, SIGTERM); // get rid of iplist
 		return reloadconfig ? 2 : 0;
 	}
-	if (o.logconerror == 1) {
+	if (o.logconerror) {
 		syslog(LOG_ERR, "%s", "Main parent process exiting.");
 	}
 	return 1;  // It is only possible to reach here with an error
