@@ -102,7 +102,7 @@ int HTTPHeader::returnCode()
 }
 
 // grab content length
-int HTTPHeader::contentLength()
+off_t HTTPHeader::contentLength()
 {
 	// code 304 - not modified - no content
 	String temp(header.front().after(" "));
@@ -110,7 +110,7 @@ int HTTPHeader::contentLength()
 		return 0;
 	if (pcontentlength != NULL) {
 		temp = pcontentlength->after(" ");
-		return temp.toInteger();
+		return temp.toOffset();
 	}
 	// no content-length header - we don't know
 	return -1;
@@ -193,7 +193,7 @@ String HTTPHeader::getContentType()
 }
 
 // does the given content type string match our headers?
-bool HTTPHeader::isContentType(String t)
+bool HTTPHeader::isContentType(const String& t)
 {
 	return getContentType().startsWith(t);
 }
@@ -237,6 +237,15 @@ std::string HTTPHeader::getAuthData()
 	return "";
 }
 
+// grab raw contents of Proxy-Authorization header without decoding
+std::string HTTPHeader::getRawAuthData()
+{
+	if (pproxyauthorization != NULL) {
+		return pproxyauthorization->after(" ").after(" ");
+	}
+	return "";
+}
+
 // do we have a non-identity content encoding? this means body is compressed
 bool HTTPHeader::isCompressed()
 {
@@ -272,7 +281,7 @@ String HTTPHeader::contentEncoding()
 // *
 
 // squid adds this so if more support it it may be useful one day
-void HTTPHeader::addXForwardedFor(std::string clientip)
+void HTTPHeader::addXForwardedFor(const std::string &clientip)
 {
 	std::string line("X-Forwarded-For: " + clientip + "\r");
 	header.push_back(String(line.c_str()));
@@ -282,7 +291,7 @@ void HTTPHeader::addXForwardedFor(std::string clientip)
 void HTTPHeader::setContentLength(int newlen)
 {
 	if (pcontentlength != NULL) {
-		(*pcontentlength) = "Content-Length: " + String(newlen);
+		(*pcontentlength) = "Content-Length: " + String(newlen) + "\r";
 	}
 }
 
@@ -337,7 +346,7 @@ String HTTPHeader::modifyEncodings(String e)
 void HTTPHeader::removeEncoding(int newlen)
 {
 	if (pcontentlength != NULL) {
-		(*pcontentlength) = "Content-Length: " + String(newlen);
+		(*pcontentlength) = "Content-Length: " + String(newlen) + "\r";
 	}
 	// this may all be overkill. since we strip everything out of the outgoing
 	// accept-encoding header that we don't support, we won't be getting anything
@@ -364,7 +373,7 @@ void HTTPHeader::removeEncoding(int newlen)
 			temp.removeWhiteSpace();
 		}
 		if (newheader.length() == 0)*/
-			(*pcontentencoding) = "X-DansGuardian-Removed: Content-Encoding";
+			(*pcontentencoding) = "X-DansGuardian-Removed: Content-Encoding\r";
 /*			else
 			header[i] = "Content-Encoding: "+newheader;
 #ifdef DGDEBUG
@@ -406,7 +415,7 @@ void HTTPHeader::setURL(String &url) {
 #ifdef DGDEBUG
 		std::cout << "setURL: header[] line changed from: " << (*phost) << std::endl;
 #endif
-		(*phost) = String("Host: ") + hostname + ":" + port + "\r";
+		(*phost) = String("Host: ") + hostname; (*phost) += ":"; (*phost) += port; (*phost) += "\r";
 #ifdef DGDEBUG
 		std::cout << " to " << (*phost) << std::endl;
 #endif
@@ -428,33 +437,28 @@ void HTTPHeader::setURL(String &url) {
 
 // Does a regexp search and replace.
 // urlRegExp Code originally from from Ton Gorter 2004
-bool HTTPHeader::urlRegExp(int filtergroup) {
-
-#ifdef DGDEBUG
-	std::cout << "Starting url reg exp replace" << std::endl;
-#endif
+bool HTTPHeader::regExp(String& line, std::deque<RegExp>& regexp_list, std::deque<String>& replacement_list) {
 	RegExp *re;
 	String replacement;
 	String repstr;
-	String oldUrl(url());
-	String newUrl;
-	bool urlmodified = false;
+	String newLine;
+	bool linemodified = false;
 	unsigned int i;
-	int j, k;
-	unsigned int s =  (*o.fg[filtergroup]).url_regexp_list_comp.size();
-	int matches, submatches;
-	int match;
-	int srcoff;
-	int nextoffset;
+	unsigned int j, k;
+	unsigned int s = regexp_list.size();
+	unsigned int matches, submatches;
+	unsigned int match;
+	unsigned int srcoff;
+	unsigned int nextoffset;
 	unsigned int matchlen;
-	int oldurllen;
+	unsigned int oldlinelen;
 
 	// iterate over our list of precompiled regexes
 	for (i = 0; i < s; i++) {
-		newUrl = "";
-		re = &((*o.fg[filtergroup]).url_regexp_list_comp[i]);
-		if (re->match(oldUrl.toCharArray())) {
-			repstr = (*o.fg[filtergroup]).url_regexp_list_rep[i];
+		newLine = "";
+		re = &(regexp_list[i]);
+		if (re->match(line.toCharArray())) {
+			repstr = replacement_list[i];
 			matches = re->numberOfMatches();
 
 			srcoff = 0;
@@ -465,7 +469,7 @@ bool HTTPHeader::urlRegExp(int filtergroup) {
 				
 				// copy next chunk of unmodified data
 				if (nextoffset > srcoff) {
-					newUrl += oldUrl.subString(srcoff, nextoffset - srcoff);
+					newLine += line.subString(srcoff, nextoffset - srcoff);
 					srcoff = nextoffset;
 				}
 
@@ -492,30 +496,63 @@ bool HTTPHeader::urlRegExp(int filtergroup) {
 				}
 				
 				// copy filled in replacement string
-				newUrl += replacement;
+				newLine += replacement;
 				srcoff += matchlen;
 				j += submatches;
 			}
-			oldurllen = oldUrl.length();
-			if (srcoff < oldurllen) {
-				newUrl += oldUrl.subString(srcoff, oldurllen - srcoff);
+			oldlinelen = line.length();
+			if (srcoff < oldlinelen) {
+				newLine += line.subString(srcoff, oldlinelen - srcoff);
 			}
-			// copy newUrl into oldUrl and continue with other regexes
-			/*setURL(*/oldUrl = newUrl/*)*/;
-			urlmodified = true;
 #ifdef DGDEBUG
-			std::cout << "URL modified!" << std::endl;
+			std::cout << "Line modified! (" << line << " -> " << newLine << ")" << std::endl;
 #endif
+			// copy newLine into line and continue with other regexes
+			line = newLine;
+			linemodified = true;
 		}
 	}
 	
-	// surely we can get away with doing this once, not every time round the loop?
-	if (urlmodified) {
-		// replace URL string in all relevant header lines
-		setURL(oldUrl);
+	return linemodified;
+}
+
+// Perform searches and replacements on URL
+bool HTTPHeader::urlRegExp(int filtergroup) {
+	// exit immediately if list is empty
+	if (not o.fg[filtergroup]->url_regexp_list_comp.size())
+		return false;
+#ifdef DGDEBUG
+	std::cout << "Starting URL reg exp replace" << std::endl;
+#endif
+	String newUrl(url());
+	if (regExp(newUrl, o.fg[filtergroup]->url_regexp_list_comp, o.fg[filtergroup]->url_regexp_list_rep)) {
+		setURL(newUrl);
+		return true;
 	}
-	
-	return urlmodified;
+	return false;
+}
+
+// Perform searches and replacements on header lines
+bool HTTPHeader::headerRegExp(int filtergroup) {
+	// exit immediately if list is empty
+	if (not o.fg[filtergroup]->header_regexp_list_comp.size())
+		return false;
+	bool result = false;
+	for (std::deque<String>::iterator i = header.begin(); i != header.end(); i++) {
+#ifdef DGDEBUG
+		std::cout << "Starting header reg exp replace: " << *i << std::endl;
+#endif
+		bool chop = false;
+		if (i->endsWith("\r"))
+		{
+			i->chop();
+			chop = true;
+		}
+		result |= regExp(*i, o.fg[filtergroup]->header_regexp_list_comp, o.fg[filtergroup]->header_regexp_list_rep);
+		if (chop)
+			i->append("\r");
+	}
+	return result;
 }
 
 // *
@@ -525,7 +562,7 @@ bool HTTPHeader::urlRegExp(int filtergroup) {
 // *
 
 // is a URL malformed?
-bool HTTPHeader::malformedURL(String url)
+bool HTTPHeader::malformedURL(const String& url)
 {
 	String host(url.after("://"));
 	if (host.contains("/"))
@@ -647,7 +684,7 @@ bool HTTPHeader::isPostUpload(Socket &peersock)
 	delete[]postdatablock;
 	return answer;*/
 
-	int cl = contentLength();
+	off_t cl = contentLength();
 	if (((cl > 0) && (cl < 14)) || (getContentType() == "application/x-www-form-urlencoded")) {
 #ifdef DGDEBUG
 		std::cout << "Based on content length/type, is not POST upload!" << std::endl;
@@ -1140,7 +1177,7 @@ bool HTTPHeader::isScanBypassURL(String * url, const char *magic, const char *cl
 
 // URL decoding (%xx)
 // uses regex pre-compiled on startup
-String HTTPHeader::decode(String s, bool decodeAll)
+String HTTPHeader::decode(const String &s, bool decodeAll)
 {
 	if (s.length() < 3) {
 		return s;
@@ -1148,7 +1185,7 @@ String HTTPHeader::decode(String s, bool decodeAll)
 #ifdef DGDEBUG
 	std::cout << "decoding url" << std::endl;
 #endif
-	if (!urldecode_re.match(s.toCharArray())) {
+	if (!urldecode_re.match(s.c_str())) {
 		return s;
 	}			// exit if not found
 #ifdef DGDEBUG
@@ -1183,10 +1220,10 @@ String HTTPHeader::decode(String s, bool decodeAll)
 }
 
 // turn %xx back into original character
-String HTTPHeader::hexToChar(String n, bool all)
+String HTTPHeader::hexToChar(const String &n, bool all)
 {
 	if (n.length() < 2) {
-		return n;
+		return String(n);
 	}
 	static char buf[2];
 	unsigned int a, b;
@@ -1203,8 +1240,7 @@ String HTTPHeader::hexToChar(String n, bool all)
 		a -= 48;
 	}
 	else {
-		n = "%" + n;
-		return n;
+		return String("%") + n;
 	}
 	if (b >= 'a' && b <= 'f') {
 		b -= 87;
@@ -1216,23 +1252,20 @@ String HTTPHeader::hexToChar(String n, bool all)
 		b -= 48;
 	}
 	else {
-		n = "%" + n;
-		return n;
+		return String("%") + n;
 	}
 	c = a * 16 + b;
-	if (all || (c >= 'a' && c <= 'z') || (c >= 'a' && c <= 'z') || (c >= '0' && c <= '9') || (c == '-')) {
+	if (all || (c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z') || (c >= '0' && c <= '9') || (c == '-')) {
 		buf[0] = c;
 		buf[1] = '\0';
-		n = buf;
+		return String(buf);
 	} else {
-		n = "%" + n;
-		return n;
+		return String("%") + n;
 	}
-	return n;
 }
 
 // decode a line of base64
-std::string HTTPHeader::decodeb64(String line)
+std::string HTTPHeader::decodeb64(const String& line)
 {				// decode a block of b64 MIME
 	long four = 0;
 	int d;
@@ -1404,7 +1437,7 @@ void HTTPHeader::out(Socket * peersock, Socket * sock, int sendflag, bool reconn
 		std::cout << "Opening tunnel for remainder of POST data" << std::endl;
 #endif
 		FDTunnel fdt;
-		int remaining = contentLength() - postdatalen;
+		off_t remaining = contentLength() - postdatalen;
 		if (remaining < 0)
 			throw runtime_error("No POST data left to send!?");
 		fdt.tunnel(*peersock, *sock, false, remaining, true);
@@ -1415,7 +1448,7 @@ void HTTPHeader::out(Socket * peersock, Socket * sock, int sendflag, bool reconn
 void HTTPHeader::discard(Socket *sock)
 {
 	static char fred[4096];
-	int cl = contentLength() - postdatalen;
+	off_t cl = contentLength() - postdatalen;
 	int rc;
 	while (cl > 0) {
 		rc = sock->readFromSocket(fred, ((cl > 4096) ? 4096 : cl), 0, timeout, false);
