@@ -29,12 +29,13 @@
 #include "RegExp.hpp"
 #include <cstdlib>
 #include <cstdio>
+#include <ctime>
 #include <unistd.h>
-#include "String.hpp"
 #include <iostream>
 #include <fstream>
 #include <sys/stat.h>
 #include <sys/time.h>
+#include <list>
 
 
 // GLOBALS
@@ -56,16 +57,17 @@ extern OptionContainer o;
 
 // Constructor - set default values
 ListContainer::ListContainer():refcount(1), parent(false), filedate(0), used(false), bannedpfiledate(0), exceptionpfiledate(0), weightedpfiledate(0),
+	blanketblock(false), blanket_ip_block(false), blanketsslblock(false), blanketssl_ip_block(false),
 	sourceisexception(false), sourcestartswith(false), sourcefilters(0), data(NULL), realgraphdata(NULL), maxchildnodes(0), graphitems(0),
-	data_length(0), data_memory(0), items(0), isSW(false), issorted(false), graphused(false), force_quick_search(0),
-	sthour(0), stmin(0), endhour(0), endmin(0), istimelimited(false)
+	data_length(0), data_memory(0), items(0), isSW(false), issorted(false), graphused(false), force_quick_search(false),
+	/*sthour(0), stmin(0), endhour(0), endmin(0),*/ istimelimited(false)
 {
 }
 
 // delete the memory block when the class is destryed
 ListContainer::~ListContainer()
 {
-	delete[]data;
+	free(data);
 	if (graphused) {
 		free(realgraphdata);
 	}
@@ -77,7 +79,7 @@ ListContainer::~ListContainer()
 // for both types of list - clear & reset all values
 void ListContainer::reset()
 {
-	delete[]data;
+	free(data);
 	if (graphused)
 		free(realgraphdata);
 	for (unsigned int i = 0; i < morelists.size(); i++) {
@@ -94,20 +96,23 @@ void ListContainer::reset()
 	issorted = false;
 	graphused = false;
 	force_quick_search = 0;
-	sthour = 0;
+	/*sthour = 0;
 	stmin = 0;
 	endhour = 0;
 	endmin = 0;
 	days = "";
-	timetag = "";
+	timetag = "";*/
 	category = "";
 	istimelimited = false;
 	combilist.clear();
 	slowgraph.clear();
 	list.clear();
+	lengthlist.clear();
 	weight.clear();
 	itemtype.clear();
+	timelimitindex.clear();
 	morelists.clear();
+	timelimits.clear();
 	listcategory.clear();
 	categoryindex.clear();
 	used = false;
@@ -131,7 +136,8 @@ bool ListContainer::previousUseItem(const char *filename, bool startswith, int f
 }
 
 // for phrase lists - read in the given file, which may be an exception list
-bool ListContainer::readPhraseList(const char *filename, bool isexception)
+// inherit category and time limits from parent
+bool ListContainer::readPhraseList(const char *filename, bool isexception, int catindex, int timeindex)
 {
 	sourcefile = filename;
 	sourceisexception = isexception;
@@ -139,7 +145,6 @@ bool ListContainer::readPhraseList(const char *filename, bool isexception)
 	String temp;  // a String for temporary manipulation
 	String line;
 	String lcat;
-	int catindex;
 	int len = getFileLength(filename);
 	if (len < 0) {
 		if (!is_daemonised) {
@@ -153,7 +158,7 @@ bool ListContainer::readPhraseList(const char *filename, bool isexception)
 		// just return
 	}
 	filedate = getFileDate(filename);
-	increaseMemoryBy(len + 1);  // Allocate some memory to hold file
+	increaseMemoryBy(len + 2);  // Allocate some memory to hold file
 	ifstream listfile(filename, ios::in);  // open the file for reading
 	if (!listfile.good()) {
 		if (!is_daemonised) {
@@ -163,7 +168,6 @@ bool ListContainer::readPhraseList(const char *filename, bool isexception)
 		return false;
 	}
 	lcat = "";
-	catindex = -1;
 	bool caseinsensitive = true;
 	while (!listfile.eof()) {	// keep going until end of file
 		getline(listfile, linebuffer);  // grab a line
@@ -176,12 +180,12 @@ bool ListContainer::readPhraseList(const char *filename, bool isexception)
 			if (caseinsensitive)
 				line.toLower();
 			if (line.startsWith("<"))
-				readPhraseListHelper(line, isexception, catindex);
+				readPhraseListHelper(line, isexception, catindex, timeindex);
 			// handle included list files
 			else if (line.startsWith(".")) {
 				temp = line.after(".include<").before(">");
 				if (temp.length() > 0) {
-					if (!readPhraseList(temp.toCharArray(), isexception)) {
+					if (!readPhraseList(temp.toCharArray(), isexception, catindex, timeindex)) {
 						listfile.close();
 						return false;
 					}
@@ -200,7 +204,6 @@ bool ListContainer::readPhraseList(const char *filename, bool isexception)
 				std::cout << "List category: " << lcat << std::endl;
 				std::cout << "Category list index: " << catindex << std::endl;
 #endif
-				continue;
 			}
 			// phrase lists can also be marked as not to be case-converted,
 			// to aid support for exotic character encodings
@@ -210,6 +213,19 @@ bool ListContainer::readPhraseList(const char *filename, bool isexception)
 #endif
 				caseinsensitive = false;
 			}
+			// Read in time tags; set timeindex to the ID of the new tag
+			else if (line.startsWith("#time: ")) {	// see if we have a time tag
+				TimeLimit tl;
+				if (!readTimeTag(&line, tl)) {
+					return false;
+				}
+				timelimits.push_back(tl);
+				timeindex = timelimits.size() - 1;
+#ifdef DGDEBUG
+				std::cout << "Found time limit on phrase list. Now have " << timelimits.size() << " limits on this list (including parents)." << std::endl;
+#endif
+				continue;
+			}
 		}
 	}
 	listfile.close();
@@ -217,7 +233,7 @@ bool ListContainer::readPhraseList(const char *filename, bool isexception)
 }
 
 // for phrase lists - helper function for readPhraseList
-void ListContainer::readPhraseListHelper(String line, bool isexception, int catindex)
+void ListContainer::readPhraseListHelper(String line, bool isexception, int catindex, int timeindex)
 {
 	// read in weighting value, if there
 	int weighting = line.after("><").before(">").toInteger();
@@ -241,20 +257,20 @@ void ListContainer::readPhraseListHelper(String line, bool isexception, int cati
 		while (line.length() > 2) {
 			line = line.after("<");
 			// combination banned, weighted, or exception
-			readPhraseListHelper2(line.before(">"), type + 11, weighting, catindex);
+			readPhraseListHelper2(line.before(">"), type + 11, weighting, catindex, timeindex);
 			line = line.after(">,");
 		}
 		// end of combi marker
-		readPhraseListHelper2("", type + 21, weighting, catindex);
+		readPhraseListHelper2("", type + 21, weighting, catindex, timeindex);
 	} else {
 		line = line.after("<").before(">");
 		// push type & weighting for this individual phrase onto combi list (not combination phrase)
-		readPhraseListHelper2(line, type, weighting, catindex);
+		readPhraseListHelper2(line, type, weighting, catindex, timeindex);
 	}
 }
 
 // for phrase lists - push phrase, type, weighting & category onto combi list
-void ListContainer::readPhraseListHelper2(String phrase, int type, int weighting, int catindex)
+void ListContainer::readPhraseListHelper2(String phrase, int type, int weighting, int catindex, int timeindex)
 {
 	// -1=exception
 	// 0=banned
@@ -267,6 +283,7 @@ void ListContainer::readPhraseListHelper2(String phrase, int type, int weighting
 	if (type > 19) {
 		combilist.push_back(-2);  // mark an end of a combi
 		combilist.push_back(type - 21);  // store the combi type
+		combilist.push_back(timeindex);  // store the combi timtime limitt
 		combilist.push_back(weighting);  // store the combi weight
 		combilist.push_back(catindex);  // store the combi category
 		return;
@@ -285,7 +302,7 @@ void ListContainer::readPhraseListHelper2(String phrase, int type, int weighting
 	}
 
 	if (type < 10) {
-		if (!addToItemListPhrase(phrase.toCharArray(), phrase.length(), type, weighting, false, catindex)) {
+		if (!addToItemListPhrase(phrase.toCharArray(), phrase.length(), type, weighting, false, catindex, timeindex)) {
 			if (!is_daemonised) {
 				std::cerr << "Duplicate phrase, dropping: " << phrase << std::endl;
 			}
@@ -297,14 +314,15 @@ void ListContainer::readPhraseListHelper2(String phrase, int type, int weighting
 
 	// must be a combi if got here
 
-	addToItemListPhrase(phrase.toCharArray(), phrase.length(), type, weighting, true, catindex);
+	addToItemListPhrase(phrase.toCharArray(), phrase.length(), type, weighting, true, catindex, timeindex);
 }
 
 // for item lists - add phrases to list proper
-bool ListContainer::addToItemListPhrase(char *s, int len, int type, int weighting, bool combi, int catindex)
+bool ListContainer::addToItemListPhrase(const char *s, int len, int type, int weighting, bool combi, int catindex, int timeindex)
 {
 	int i;
 	list.push_back(data_length);
+	lengthlist.push_back(len);
 	for (i = 0; i < len; i++) {
 		data[data_length + i] = s[i];
 	}
@@ -318,6 +336,7 @@ bool ListContainer::addToItemListPhrase(char *s, int len, int type, int weightin
 	weight.push_back(weighting);
 	itemtype.push_back(type);
 	categoryindex.push_back(catindex);
+	timelimitindex.push_back(timeindex);
 	return true;
 }
 
@@ -364,7 +383,7 @@ bool ListContainer::readItemList(const char *filename, bool startswith, int filt
 		return true;  // its blank - perhaps due to webmin editing
 		// just return
 	}
-	increaseMemoryBy(len + 1);  // Allocate some memory to hold file
+	increaseMemoryBy(len + 2);  // Allocate some memory to hold file
 	// The plus one is to cope with files not ending in a new line
 	ifstream listfile(filename, ios::in);
 	if (!listfile.good()) {
@@ -385,7 +404,7 @@ bool ListContainer::readItemList(const char *filename, bool startswith, int filt
 		//item lists (URLs, domains) can be both categorised and time-limited
 		if (linebuffer[0] == '#') {
 			if (temp.startsWith("#time: ")) {	// see if we have a time tag
-				if (!readTimeTag(&temp)) {
+				if (!readTimeTag(&temp, listtimelimit)) {
 					return false;
 				}
 				continue;
@@ -400,6 +419,21 @@ bool ListContainer::readItemList(const char *filename, bool startswith, int filt
 			continue;  // it's a comment
 		}
 
+		// blanket block flags
+		if (linebuffer == "**") {
+			blanketblock = true;
+			continue;
+		} else if (linebuffer == "*ip") {
+			blanket_ip_block = true;
+			continue;
+		} else if (linebuffer == "**s") {
+			blanketsslblock = true;
+			continue;
+		} else if (linebuffer == "**ips") {
+			blanketssl_ip_block = true;
+			continue;
+		}
+
 		if (temp.contains("#")) {
 			temp = temp.before("#");  // tidy up
 		}
@@ -420,7 +454,7 @@ bool ListContainer::readItemList(const char *filename, bool startswith, int filt
 			temp = temp.after("ftp://");  // tidy up
 		}
 		if (filters == 1) {	// remove port addresses
-			if (temp.contains(":")) {	// quicker than full regexp
+			if (temp.before("/").contains(":")) {	// quicker than full regexp
 				if (re.match(temp.toCharArray())) {
 					hostname = temp.before(":");
 					url = temp.after("/");
@@ -431,8 +465,8 @@ bool ListContainer::readItemList(const char *filename, bool startswith, int filt
 		if (filters != 32) {
 			temp.toLower();  // tidy up - but don't make regex lists lowercase!
 		}
-		addToItemList(temp.toCharArray(), temp.length());  // add to unsorted
-		// list
+		if (temp.length() > 0)
+			addToItemList(temp.toCharArray(), temp.length());  // add to unsorted list
 	}
 	listfile.close();
 	return true;  // sucessful read
@@ -455,7 +489,7 @@ bool ListContainer::readAnotherItemList(const char *filename, bool startswith, i
 }
 
 // for item lists - is this item in the list?
-bool ListContainer::inList(char *string)
+bool ListContainer::inList(const char *string)
 {
 	if (findInList(string) != NULL) {
 		return true;
@@ -464,7 +498,7 @@ bool ListContainer::inList(char *string)
 }
 
 // for item lists - is an item in the list that ends with this string?
-bool ListContainer::inListEndsWith(char *string)
+bool ListContainer::inListEndsWith(const char *string)
 {
 	if (isNow()) {
 		if (items > 0) {
@@ -486,7 +520,7 @@ bool ListContainer::inListEndsWith(char *string)
 }
 
 // for item lists - is an item in the list that starts with this string?
-bool ListContainer::inListStartsWith(char *string)
+bool ListContainer::inListStartsWith(const char *string)
 {
 	if (isNow()) {
 		if (items > 0) {
@@ -508,7 +542,7 @@ bool ListContainer::inListStartsWith(char *string)
 }
 
 // find pointer to the part of the data array containing this string
-char *ListContainer::findInList(char *string)
+char *ListContainer::findInList(const char *string)
 {
 	if (isNow()) {
 		if (items > 0) {
@@ -536,7 +570,7 @@ char *ListContainer::findInList(char *string)
 }
 
 // find an item in the list which starts with this
-char *ListContainer::findStartsWith(char *string)
+char *ListContainer::findStartsWith(const char *string)
 {
 	if (isNow()) {
 		if (items > 0) {
@@ -558,7 +592,7 @@ char *ListContainer::findStartsWith(char *string)
 	return NULL;
 }
 
-char *ListContainer::findStartsWithPartial(char *string)
+char *ListContainer::findStartsWithPartial(const char *string)
 {
 	if (isNow()) {
 		if (items > 0) {
@@ -585,7 +619,7 @@ char *ListContainer::findStartsWithPartial(char *string)
 	return NULL;
 }
 
-char *ListContainer::findEndsWith(char *string)
+char *ListContainer::findEndsWith(const char *string)
 {
 	if (isNow()) {
 		if (items > 0) {
@@ -607,21 +641,137 @@ char *ListContainer::findEndsWith(char *string)
 	return NULL;
 }
 
+// For phrase lists - grab the text, score and type of a given phrase, based on item number within list
 std::string ListContainer::getItemAtInt(int index)
 {
-	std::string s(data + list[index]);
+	std::string s(data + list[index], lengthlist[index]);
 	return s;
 }
-
 int ListContainer::getWeightAt(unsigned int index)
 {
 	return weight[index];
 }
-
 int ListContainer::getTypeAt(unsigned int index)
 {
 	return itemtype[index];
 }
+// Phrase lists - check whether the current time is within the limit imposed upon the given phrase
+bool ListContainer::checkTimeAt(unsigned int index)
+{
+	if (timelimitindex[index] == -1) {
+		return true;
+	}
+	return isNow(timelimitindex[index]);
+}
+bool ListContainer::checkTimeAtD(int index)
+{
+	if (index == -1) {
+		return true;
+	}
+	return isNow(index);
+}
+
+struct lessThanEWF: public std::binary_function<const unsigned int&, const unsigned int&, bool>
+{
+	bool operator()(const unsigned int& aoff, const unsigned int& boff)
+	{
+		const char* a = data + aoff;
+		const char* b = data + boff;
+		int alen = strlen(a);
+		int blen = strlen(b);
+		int apos = alen - 1;
+		int bpos = blen - 1;
+		for (int maxlen = (alen < blen ? alen : blen); maxlen > 0; apos--, bpos--, maxlen--)
+			if (a[apos] > b[bpos])
+				return true;
+			else if (a[apos] < b[bpos])
+				return false;
+		if (alen > blen)
+			return true;
+		else if (alen < blen)
+			return false;
+		return true;  // both equal
+	};
+	char *data;
+};
+
+struct lessThanSWF: public std::binary_function<const unsigned int&, const unsigned int&, bool>
+{
+	bool operator()(const unsigned int& aoff, const unsigned int& boff)
+	{
+		const char* a = data + aoff;
+		const char* b = data + boff;
+		int alen = strlen(a);
+		int blen = strlen(b);
+		int maxlen = alen < blen ? alen : blen;
+		for (int i = 0; i < maxlen; i++)
+			if (a[i] > b[i])
+				return true;
+			else if (a[i] < b[i])
+				return false;
+		if (alen > blen)
+			return true;
+		else if (alen < blen)
+			return false;
+		return true;  // both equal
+	};
+	char *data;
+};
+
+/*struct lessThanEW: public std::binary_function<const unsigned int&, const unsigned int&, bool>
+{
+	bool operator()(const unsigned int& aoff, const unsigned int& boff)
+	{
+		const char* a = data + aoff;
+		const char* b = data + boff;
+		int alen = strlen(a);
+		int blen = strlen(b);
+		int apos = alen - 1;
+		int bpos = blen - 1;
+		for (int maxlen = (alen < blen ? alen : blen); maxlen > 0; apos--, bpos--, maxlen--)
+			if (a[apos] > b[bpos])
+				return true;
+			else if (a[apos] < b[bpos])
+				return false;
+		if (blen > alen)
+			return false;
+		return true;  // both equal
+	};
+	char *data;
+};
+
+struct lessThanSW: public std::binary_function<const unsigned int&, const unsigned int&, bool>
+{
+	bool operator()(const unsigned int& aoff, const unsigned int& boff)
+	{
+		const char* a = data + aoff;
+		const char* b = data + boff;
+		int alen = strlen(a);
+		int blen = strlen(b);
+		int maxlen = alen < blen ? alen : blen;
+		for (int i = 0; i < maxlen; i++)
+			if (a[i] > b[i])
+				return true;
+			else if (a[i] < b[i])
+				return false;
+		
+		// if the URLs didn't match and the one the user is browsing to is longer
+		// than what we just compared against, we need to compare against longer URLs,
+		// but only if the next character is actually part of a folder name rather than a separator.
+		// ('cos if it *is* a separator, it doesn't matter that the overall URL is longer, the
+		// beginning of it matches a banned URL.)
+		if ((alen > blen) && !(a[blen] == '/' || a[blen]=='?' || a[blen]=='&' || a[blen]=='='))
+			return true;
+
+		// if the banned URL is longer than the URL we're checking, the two
+		// can't possibly match.
+		else if (blen > alen)
+			return false;
+
+		return true;  // both equal
+	};
+	char *data;
+};*/
 
 void ListContainer::doSort(const bool startsWith)
 {				// sort by ending of line
@@ -630,9 +780,15 @@ void ListContainer::doSort(const bool startsWith)
 	if (items < 2 || issorted)
 		return;
 	if (startsWith)
-		quicksort(&ListContainer::greaterThanSWF, 0, items - 1);
-	else
-		quicksort(&ListContainer::greaterThanEWF, 0, items - 1);
+	{
+		lessThanSWF lts;
+		lts.data = data;
+		std::sort(list.begin(), list.end(), lts);
+	} else {
+		lessThanEWF lte;
+		lte.data = data;
+		std::sort(list.begin(), list.end(), lte);
+	}
 	isSW = startsWith;
 	issorted = true;
 	return;
@@ -670,8 +826,8 @@ bool ListContainer::createCacheFile()
 		f += ">\n";
 		listfile.write(f.toCharArray(), f.length());
 	}
-	if (timetag.length() > 10) {	// no point in writing corrupt short one
-		f = timetag;
+	if (listtimelimit.timetag.length() > 10) {	// no point in writing corrupt short one
+		f = listtimelimit.timetag;
 		f += "\n";
 		listfile.write(f.toCharArray(), f.length());
 	}
@@ -693,29 +849,74 @@ bool ListContainer::createCacheFile()
 	return true;
 }
 
-void ListContainer::makeGraph(int fqs)
+bool ListContainer::makeGraph(bool fqs)
 {
 	force_quick_search = fqs;
 	if (data_length == 0)
-		return;
+		return true;
 	int i;
-	if (force_quick_search == 1) {
+	// Quick search has been forced on - put all items on the "slow" list and be done with it
+	if (force_quick_search) {
 		for (i = 0; i < items; i++) {
-			slowgraph.push_back(i);
+			// Check to see if the item is a duplicate
+			std::string thisphrase = getItemAtInt(i);
+			bool found = false;
+			unsigned int foundindex = 0;
+			for (std::vector<unsigned int>::iterator j = slowgraph.begin(); j != slowgraph.end(); j++) {
+				if (getItemAtInt(*j) == thisphrase) {
+					found = true;
+					foundindex = *j;
+					break;
+				}
+			}
+			if (!found) {
+				// Not a duplicate - store it
+				slowgraph.push_back(i);
+			} else {
+				// Duplicate - resolve the collision
+				// 
+				// Existing entry must be a combi AND
+				// new entry is not a combi so we overwrite the
+				// existing values as combi values and types are
+				// stored in the combilist
+				// OR
+				// both are weighted phrases and the new phrase is higher weighted
+				// OR
+				// the existing phrase is weighted and the new phrase is banned
+				// OR
+				// new phrase is an exception; exception phrases take precedence
+				if ((itemtype[foundindex] > 9 && itemtype[i] < 10) ||
+					(itemtype[foundindex] == 1 && itemtype[i] == 1 && (weight[i] > weight[foundindex])) ||
+					(itemtype[foundindex] == 1 && itemtype[i] == 0) ||
+					itemtype[i] == -1)
+				{
+					itemtype[foundindex] = itemtype[i];
+					weight[foundindex] = weight[i];
+					categoryindex[foundindex] = categoryindex[i];
+					timelimitindex[foundindex] = timelimitindex[i];
+				}
+			}
 		}
-		return;
+		return true;
 	}
 	std::string s;
 	std::string lasts;
 	graphused = true;
 
 #ifdef DGDEBUG
-	std::cout << "Bytes needed for phrase tree in worst-case scenario: " << (sizeof(int) * ((GRAPHENTRYSIZE * data_length) + ROOTOFFSET)) << std::endl;
+	std::cout << "Bytes needed for phrase tree in worst-case scenario: " << (sizeof(int) * ((GRAPHENTRYSIZE * data_length) + ROOTOFFSET))
+		<< ", starting off with allocation of " <<  (sizeof(int) * ((GRAPHENTRYSIZE * ((data_length / 3) + 1)) + ROOTOFFSET)) << std::endl;
 	prolificroot = false;
 	secondmaxchildnodes = 0;
 #endif
 
-	realgraphdata = (int*) calloc((GRAPHENTRYSIZE * data_length) + ROOTOFFSET, sizeof(int));
+	// Make a conservative guess at how much memory will be needed - call realloc() as necessary to change what is actually taken
+	current_graphdata_size = (GRAPHENTRYSIZE * ((data_length / 3) + 1)) + ROOTOFFSET;
+	realgraphdata = (int*) calloc(current_graphdata_size, sizeof(int));
+	if (realgraphdata == NULL) {
+		syslog(LOG_ERR, "Cannot allocate memory for phrase tree: %s", strerror(errno));
+		return false;
+	}
 	graphitems++;
 	std::deque<unsigned int> sizelist;
 
@@ -725,7 +926,7 @@ void ListContainer::makeGraph(int fqs)
 	graphSizeSort(0, items - 1, &sizelist);
 
 	for (i = 0; i < items; i++) {
-		graphAdd(data + list[sizelist[i]], 0, sizelist[i]);
+		graphAdd(String(data + list[sizelist[i]], lengthlist[sizelist[i]]), 0, sizelist[i]);
 	}
 
 #ifdef DGDEBUG
@@ -736,6 +937,10 @@ void ListContainer::makeGraph(int fqs)
 #endif
 
 	realgraphdata = (int*) realloc(realgraphdata, sizeof(int) * ((GRAPHENTRYSIZE * graphitems) + ROOTOFFSET));
+	if (realgraphdata == NULL) {
+		syslog(LOG_ERR, "Cannot reallocate memory for phrase tree: %s", strerror(errno));
+		return false;
+	}
 
 	int ml = realgraphdata[2];
 	int branches;
@@ -752,6 +957,7 @@ void ListContainer::makeGraph(int fqs)
 			realgraphdata[2]--;
 		}
 	}
+	return true;
 }
 
 void ListContainer::graphSizeSort(int l, int r, std::deque<unsigned int> *sizelist)
@@ -839,20 +1045,45 @@ void ListContainer::graphCopyNodePhrases(unsigned int pos)
 		graphCopyNodePhrases(graphdata[pos * GRAPHENTRYSIZE + 4 + i]);
 	}
 	bool found = false;
-	int size = slowgraph.size();
+	unsigned int foundindex = 0;
 	unsigned int phrasenumber = graphdata[pos * GRAPHENTRYSIZE + 3];
-	for (i = 0; i < size; i++) {
-		if (slowgraph[i] == phrasenumber) {
+	std::string thisphrase = getItemAtInt(phrasenumber);
+	for (std::vector<unsigned int>::iterator i = slowgraph.begin(); i != slowgraph.end(); i++) {
+		if (getItemAtInt(*i) == thisphrase) {
 			found = true;
+			foundindex = *i;
 			break;
 		}
 	}
 	if (!found) {
 		slowgraph.push_back(phrasenumber);
+	} else {
+		// Duplicate - resolve the collision
+		// 
+		// Existing entry must be a combi AND
+		// new entry is not a combi so we overwrite the
+		// existing values as combi values and types are
+		// stored in the combilist
+		// OR
+		// both are weighted phrases and the new phrase is higher weighted
+		// OR
+		// the existing phrase is weighted and the new phrase is banned
+		// OR
+		// new phrase is an exception; exception phrases take precedence
+		if ((itemtype[foundindex] > 9 && itemtype[phrasenumber] < 10) ||
+			(itemtype[foundindex] == 1 && itemtype[phrasenumber] == 1 && (weight[phrasenumber] > weight[foundindex])) ||
+			(itemtype[foundindex] == 1 && itemtype[phrasenumber] == 0) ||
+			itemtype[phrasenumber] == -1)
+		{
+			itemtype[foundindex] = itemtype[phrasenumber];
+			weight[foundindex] = weight[phrasenumber];
+			categoryindex[foundindex] = categoryindex[phrasenumber];
+			timelimitindex[foundindex] = timelimitindex[phrasenumber];
+		}
 	}
 }
 
-int ListContainer::bmsearch(char *file, int fl, std::string s)
+int ListContainer::bmsearch(char *file, int fl, const std::string& s)
 {
 	int pl = s.length();
 	if (fl < pl)
@@ -921,31 +1152,42 @@ int ListContainer::bmsearch(char *file, int fl, std::string s)
 // Format of the data is each entry has GRAPHENTRYSIZE int values with format of:
 // [letter][last letter flag][num links][from phrase][link0][link1]...
 
-void ListContainer::graphSearch(std::deque<unsigned int>& result, char *doc, int len)
+void ListContainer::graphSearch(std::map<std::string, std::pair<unsigned int, unsigned int> >& result, char *doc, int len)
 {
 	int i, j, k;
-	int sl;
-	int ppos;
-	int * graphdata = realgraphdata;
+	std::map<std::string, std::pair<unsigned int, unsigned int> >::iterator existingitem;
 	
 	//do standard quick search on short branches (or everything, if force_quick_search is on)
-	sl = slowgraph.size();
-	for (i = 0; i < sl; i++) {
-		ppos = slowgraph[i];
-		j = bmsearch(doc, len, getItemAtInt(ppos));
+	for (std::vector<unsigned int>::iterator i = slowgraph.begin(); i != slowgraph.end(); i++) {
+		std::string phrase = getItemAtInt(*i);
+		j = bmsearch(doc, len, phrase);
 		for (k = 0; k < j; k++) {
-			result.push_back(ppos);
+			existingitem = result.find(phrase);
+			if (existingitem == result.end()) {
+				result[phrase] = std::pair<unsigned int, unsigned int>(*i, 1);
+			} else {
+				existingitem->second.second++;
+			}
 		}
 	}
 	
-	if (force_quick_search == 1 || graphitems == 0) {
+	if (force_quick_search || graphitems == 0) {
+#ifdef DGDEBUG
+		std::cout << "Map (quicksearch) start" << std::endl;
+		for (std::map<std::string, std::pair<unsigned int, unsigned int> >::iterator i = result.begin(); i != result.end(); i++) {
+			std::cout << "Map: " << i->first << " " << i->second.second << std::endl;
+		}
+		std::cout << "Map (quicksearch) end" << std::endl;
+#endif
 		return;
 	}
 	
+	int sl;
+	int ppos;
+	int currnode;
+	int * graphdata = realgraphdata;
 	int ml;
-	int *stack = new int[maxchildnodes];
-	int stacksize = 0;
-	unsigned char p;
+	char p;
 	int pos;
 	int depth;
 	// number of links from root node to first letter of phrase
@@ -956,6 +1198,7 @@ void ListContainer::graphSearch(std::deque<unsigned int>& result, char *doc, int
 		for (j = 4; j < ml; j++) {
 			// grab the endpoint of this link
 			pos = realgraphdata[j];
+			sl = 0;
 
 			// now comes the main graph search!
 			// this is basically a depth-first tree search
@@ -976,21 +1219,26 @@ void ListContainer::graphSearch(std::deque<unsigned int>& result, char *doc, int
 					// is this graph node marked as being the end of a phrase?
 					if (graphdata[ppos + 1] == 1) {
 						// it is, so store the pointer to the matched phrase.
-						result.push_back(graphdata[ppos + 3]);
+						std::string phrase = getItemAtInt(graphdata[ppos + 3]);
+						existingitem = result.find(phrase);
+						if (existingitem == result.end()) {
+							result[phrase] = std::pair<unsigned int, unsigned int>(graphdata[ppos + 3], 1);
+						} else {
+							existingitem->second.second++;
+						}
+#ifdef DGDEBUG
+						std::cout << "Found this phrase: " << phrase << std::endl;
+#endif
 					}
 					// grab this node's number of children
 					sl = graphdata[ppos + 2];
 					if (sl > 0) {
-						// add the node's children (bar the first) to the stack for later examination
-						// and signal that we're moving along the current string by one.
-						depth++;
-						stacksize=0;
-						for (k = 1; k < sl; k++) {
-							stack[stacksize++] = graphdata[ppos + 4 + k];
-						}
+						// this is now the node we're interested in looking at the children of
+						currnode = ppos;
 						// zip straight to the first child of the matched node
 						// (this is the magic that makes it depth first)
 						pos = graphdata[ppos + 4];
+						depth++;
 						continue;
 					}
 					// if we just matched a node that has no children,
@@ -999,19 +1247,15 @@ void ListContainer::graphSearch(std::deque<unsigned int>& result, char *doc, int
 					else break;
 				}
 
-				if (stacksize > 0) {
-					// if we get here, we have discounted one branch, but
-					// we still have more branches to examine at this level.
-					// but do we ever actually need to backtrack here?
-					// after all, surely a character will only ever match
-					// one node at any given level. so perhaps *only* store
-					// a stack of children from the most recently matched node,
-					// so that this code will never initiate backtracking.
-					// the stack would then be allowed to be reduced to 63,
-					// since we only have max GRAPHENTRYSIZE children on a node, and
-					// we'd set stacksize to 0 before we do any adding, to
-					// effectively clear it beforehand.
-					pos = stack[--stacksize];
+				if ((--sl) > 0) {
+					// if we get here, we have discounted one child, but
+					// we still have more children to examine from the last matched node.
+					// we don't keep more than one current interesting node - no backtracking
+					// is necessary, as there is only ever one occurrence of a given character as
+					// a branch of a given node.  backtracking would therefore never
+					// trigger a match down a different route than has been taken thus far, so
+					// don't bother.
+					pos = graphdata[currnode + 4 + (graphdata[currnode + 2] - sl)];
 					continue;
 				}
 				// if we get here, we've discounted all branches at this depth, and the search is over.
@@ -1019,16 +1263,22 @@ void ListContainer::graphSearch(std::deque<unsigned int>& result, char *doc, int
 			}
 		}
 	}
-	delete[]stack;
+#ifdef DGDEBUG
+	std::cout << "Map start" << std::endl;
+	for (std::map<std::string, std::pair<unsigned int, unsigned int> >::iterator i = result.begin(); i != result.end(); i++) {
+		std::cout << "Map: " << i->first << " " << i->second.second << std::endl;
+	}
+	std::cout << "Map end" << std::endl;
+#endif
 }
 
-void ListContainer::graphAdd(String s, int inx, int item)
+void ListContainer::graphAdd(String s, const int inx, int item)
 {
 	unsigned char p = s.charAt(0);
 	unsigned char c;
 	bool found = false;
 	String t;
-	int i, px, it;
+	int i, px;
 	int numlinks;
 	int *graphdata;
 	int *graphdata2 = realgraphdata + ROOTOFFSET;
@@ -1059,20 +1309,34 @@ void ListContainer::graphAdd(String s, int inx, int item)
 			if (px == 1) {
 				// the exact phrase is already there
 				px = graphdata2[(graphdata[inx * GRAPHENTRYSIZE + 4 + i]) * GRAPHENTRYSIZE + 3];
-				it = itemtype[px];
 
-				if ((it > 9 && itemtype[item] < 10) || itemtype[item] == -1) {
+				// -1=exception
+				// 0=banned
+				// 1=weighted
+				// 10 = combination exception
+				// 11 = combination banned
+				// 12 = combination weighted
+				// 20,21,22 = end of combi marker
+				if ((itemtype[px] > 9 && itemtype[item] < 10) ||
+					(itemtype[px] == 1 && itemtype[item] == 1 && (weight[item] > weight[px])) ||
+					(itemtype[px] == 1 && itemtype[item] == 0) ||
+					itemtype[item] == -1)
+				{
 					// exists as a combi entry already
 					// if got here existing entry must be a combi AND
 					// new entry is not a combi so we overwrite the
 					// existing values as combi values and types are
 					// stored in the combilist
 					// OR
-					// its a an exception
-					// exception phrases take precedence
+					// both are weighted phrases and the new phrase is higher weighted
+					// OR
+					// the existing phrase is weighted and the new phrase is banned
+					// OR
+					// new phrase is an exception; exception phrases take precedence
 					itemtype[px] = itemtype[item];
 					weight[px] = weight[item];
 					categoryindex[px] = categoryindex[item];
+					timelimitindex[px] = timelimitindex[item];
 				}
 			}
 		}
@@ -1081,6 +1345,22 @@ void ListContainer::graphAdd(String s, int inx, int item)
 	if (!found) {
 		i = graphitems;
 		graphitems++;
+		// Reallocate memory if we're running out
+		if (current_graphdata_size < ((GRAPHENTRYSIZE * graphitems) + ROOTOFFSET)) {
+			int new_current_graphdata_size = (GRAPHENTRYSIZE * (graphitems + 256)) + ROOTOFFSET;
+			realgraphdata = (int*) realloc(realgraphdata, sizeof(int) * new_current_graphdata_size);
+			if (realgraphdata == NULL) {
+				syslog(LOG_ERR, "Cannot reallocate memory for phrase tree: %s", strerror(errno));
+				exit(1);
+			}
+			memset(realgraphdata + current_graphdata_size, 0, sizeof(int) * (new_current_graphdata_size - current_graphdata_size));
+			current_graphdata_size = new_current_graphdata_size;
+			graphdata2 = realgraphdata + ROOTOFFSET;
+			if (inx == 0)
+				graphdata = realgraphdata;
+			else
+				graphdata = realgraphdata + ROOTOFFSET;
+		}
 		numlinks = graphdata[inx * GRAPHENTRYSIZE + 2];
 		if ((inx == 0) ? ((numlinks + 1) > MAXROOTLINKS) : ((numlinks +1) > MAXLINKS)) {
 			syslog(LOG_ERR, "Cannot load phraselists from this many languages/encodings simultaneously. (more than %d links from this node! [1])", (inx == 0) ? MAXROOTLINKS : MAXLINKS);
@@ -1126,6 +1406,22 @@ void ListContainer::graphAdd(String s, int inx, int item)
 			graphdata2[i * GRAPHENTRYSIZE + 4 + numlinks] = i + 1;
 			i++;
 			graphitems++;
+			// Reallocate memory if we're running out
+			if (current_graphdata_size < ((GRAPHENTRYSIZE * graphitems) + ROOTOFFSET)) {
+				int new_current_graphdata_size = (GRAPHENTRYSIZE * (graphitems + 256)) + ROOTOFFSET;
+				realgraphdata = (int*) realloc(realgraphdata, sizeof(int) * new_current_graphdata_size);
+				if (realgraphdata == NULL) {
+					syslog(LOG_ERR, "Cannot reallocate memory for phrase tree: %s", strerror(errno));
+					exit(1);
+				}
+				memset(realgraphdata + current_graphdata_size, 0, sizeof(int) * (new_current_graphdata_size - current_graphdata_size));
+				current_graphdata_size = new_current_graphdata_size;
+				graphdata2 = realgraphdata + ROOTOFFSET;
+				if (inx == 0)
+					graphdata = realgraphdata;
+				else
+					graphdata = realgraphdata + ROOTOFFSET;
+			}
 			p = s.charAt(0);
 			graphdata2[i * GRAPHENTRYSIZE] = p;
 			graphdata2[i * GRAPHENTRYSIZE + 3] = item;
@@ -1135,58 +1431,6 @@ void ListContainer::graphAdd(String s, int inx, int item)
 	}
 }
 
-// quicksort with 3 way partitioning sorted by the end of the line
-void ListContainer::quicksort(int (ListContainer::*comparitor)(const char* a, const char* b), int l, int r)
-{
-	if (r <= l)
-		return;
-	unsigned int e;
-	int k;
-	char *v = data + list[r];
-	int i = l - 1, j = r, p = i, q = r;
-	for (;;) {
-		while ((this->*comparitor)(data + list[++i], v) > 0);
-		while ((this->*comparitor)(v, data + list[--j]) > 0) {
-			if (j == l)
-				break;
-		}
-		if (i >= j)
-			break;
-		e = list[i];
-		list[i] = list[j];
-		list[j] = e;
-		if ((this->*comparitor)(v, data + list[i]) == 0) {
-			p++;
-			e = list[p];
-			list[p] = list[i];
-			list[i] = e;
-		}
-		if ((this->*comparitor)(v, data + list[j]) == 0) {
-			q--;
-			e = list[q];
-			list[q] = list[j];
-			list[j] = e;
-		}
-	}
-	e = list[i];
-	list[i] = list[r];
-	list[r] = e;
-	j = i - 1;
-	i++;
-	for (k = l; k <= p; k++, j--) {
-		e = list[k];
-		list[k] = list[j];
-		list[j] = e;
-	}
-	for (k = r - 1; k >= q; k--, i++) {
-		e = list[k];
-		list[k] = list[i];
-		list[i] = e;
-	}
-	quicksort(comparitor, l, j);
-	quicksort(comparitor, i, r);
-}
-
 bool ListContainer::readProcessedItemList(const char *filename, bool startswith, int filters)
 {
 #ifdef DGDEBUG
@@ -1236,7 +1480,7 @@ bool ListContainer::readProcessedItemList(const char *filename, bool startswith,
 		else if (linebuffer[0] == '#') {
 			temp = linebuffer.c_str();
 			if (temp.startsWith("#time: ")) {	// see if we have a time tag
-				if (!readTimeTag(&temp)) {
+				if (!readTimeTag(&temp, listtimelimit)) {
 					return false;
 				}
 				continue;
@@ -1250,11 +1494,26 @@ bool ListContainer::readProcessedItemList(const char *filename, bool startswith,
 			}
 			continue;  // it's a comment man
 		}
+		// blanket block flags
+		if (linebuffer == "**") {
+			blanketblock = true;
+			continue;
+		} else if (linebuffer == "*ip") {
+			blanket_ip_block = true;
+			continue;
+		} else if (linebuffer == "**s") {
+			blanketsslblock = true;
+			continue;
+		} else if (linebuffer == "**ips") {
+			blanketssl_ip_block = true;
+			continue;
+		}
 		slen = linebuffer.length();
 		if (slen < 3)
 			continue;
 		i = slen - 1;
 		list.push_back(data_length);
+		lengthlist.push_back(slen);
 		for (; i >= 0; i--) {
 			data[data_length + i] = linebuffer[i];
 		}
@@ -1266,10 +1525,11 @@ bool ListContainer::readProcessedItemList(const char *filename, bool startswith,
 	return true;
 }
 
-void ListContainer::addToItemList(char *s, int len)
+void ListContainer::addToItemList(const char *s, int len)
 {
 	int i;
 	list.push_back(data_length);
+	lengthlist.push_back(len);
 	for (i = 0; i < len; i++) {
 		data[data_length + i] = s[i];
 	}
@@ -1394,14 +1654,16 @@ bool ListContainer::isCacheFileNewer(const char *filename)
 void ListContainer::increaseMemoryBy(int bytes)
 {
 	if (data_memory > 0) {
-		char *temp = new char[data_memory + bytes];  // replacement store
+		/*char *temp = new char[data_memory + bytes];  // replacement store
 		memcpy(temp, data, data_length);
 		delete[]data;
-		data = temp;
+		data = temp;*/
+		data = (char*) realloc(data, (data_memory + bytes) * sizeof(char));
+		memset(data + data_memory, 0, bytes * sizeof(char));
 		data_memory = data_memory + bytes;
 	} else {
-		delete[]data;
-		data = new char[bytes];
+		free(data);
+		data = (char*) calloc(bytes, sizeof(char));
 		data_memory = bytes;
 	}
 }
@@ -1446,7 +1708,7 @@ bool ListContainer::upToDate()
 	return true;
 }
 
-bool ListContainer::readTimeTag(String * tag)
+bool ListContainer::readTimeTag(String * tag, TimeLimit& tl)
 {
 #ifdef DGDEBUG
 	std::cout << "Found a time tag" << std::endl;
@@ -1498,18 +1760,25 @@ bool ListContainer::readTimeTag(String * tag)
 		return false;
 	}
 	istimelimited = true;
-	sthour = tsthour;
-	stmin = tstmin;
-	endhour = tendhour;
-	endmin = tendmin;
-	days = tdays;
-	timetag = (*tag);
+	tl.sthour = tsthour;
+	tl.stmin = tstmin;
+	tl.endhour = tendhour;
+	tl.endmin = tendmin;
+	tl.days = tdays;
+	tl.timetag = (*tag);
 	return true;
 }
 
-bool ListContainer::isNow()
+
+// Returns true if the current time is within the limits specified on this list.
+// For phrases, the time limit list index must be passed in -
+// included lists don't have their own ListContainer, so time limits are stored differently.
+bool ListContainer::isNow(int index)
 {
-	// returns true if the current time is within the limits specified on this list
+	TimeLimit& tl = listtimelimit;
+	if (index > -1) {
+		tl = timelimits[index];
+	}
 	if (!istimelimited) {
 		return true;
 	}
@@ -1528,8 +1797,8 @@ bool ListContainer::isNow()
 	wday--;
 	unsigned char cday = '0' + wday;
 	bool matchday = false;
-	for (int i = 0; i < days.length(); i++) {
-		if (days[i] == cday) {
+	for (unsigned int i = 0; i < tl.days.length(); i++) {
+		if (tl.days[i] == cday) {
 			matchday = true;
 			break;
 		}
@@ -1537,24 +1806,24 @@ bool ListContainer::isNow()
 	if (!matchday) {
 		return false;
 	}
-	if (hour < sthour) {
+	if (hour < tl.sthour) {
 		return false;
 	}
-	if (hour > endhour) {
+	if (hour > tl.endhour) {
 		return false;
 	}
-	if (hour == sthour) {
-		if (min < stmin) {
+	if (hour == tl.sthour) {
+		if (min < tl.stmin) {
 			return false;
 		}
 	}
-	if (hour == endhour) {
-		if (min > endmin) {
+	if (hour == tl.endhour) {
+		if (min > tl.endmin) {
 			return false;
 		}
 	}
 #ifdef DGDEBUG
-	std::cout << "time match " << sthour << ":" << stmin << "-" << endhour << ":" << endmin << " " << hour << ":" << min << " " << sourcefile << std::endl;
+	std::cout << "time match " << tl.sthour << ":" << tl.stmin << "-" << tl.endhour << ":" << tl.endmin << " " << hour << ":" << min << " " << sourcefile << std::endl;
 #endif
 	return true;
 }
