@@ -26,7 +26,6 @@
 
 #include <syslog.h>
 #include <algorithm>
-#include <deque>
 
 
 // GLOBALS 
@@ -45,12 +44,11 @@ extern RegExp absurl_re, relurl_re;
 // operator so that the STL sort algorithm can be applied to lists of these.
 class listent {
 public:
-	listent(const int& c, const int& w, String& s) {
+	listent() {};
+	listent(const int& w, String& s) {
 		weight = w;
-		cat = c;
 		string = s;
 	};
-	int cat;
 	int weight;
 	String string;
 	int operator < (const listent &a) const {
@@ -72,10 +70,10 @@ void NaughtyFilter::reset()
 {
 	isItNaughty = false;
 	isException = false;
-	/*filtergroup = 0;
+	filtergroup = 0;
 	whatIsNaughty = "";
 	whatIsNaughtyLog = "";
-	whatIsNaughtyCategories = "";*/
+	whatIsNaughtyCategories = "";
 	usedisplaycats = false;
 	naughtiness = 0;
 }
@@ -89,7 +87,7 @@ void NaughtyFilter::checkme(DataBuffer *body, String &url, String &domain)
 	
 	// check PICS now - not dependent on case, hex decoding, etc.
 	// as only sites which play by the rules will self-rate
-	if ((*o.fg[filtergroup]).enable_PICS == 1) {
+	if ((*o.fg[filtergroup]).enable_PICS) {
 #ifdef DGDEBUG
 		std::cout << "PICS is enabled" << std::endl;
 #endif
@@ -110,7 +108,7 @@ void NaughtyFilter::checkme(DataBuffer *body, String &url, String &domain)
 	// Hex decode content if desired
 	// Do this now, as it's not especially case-sensitive,
 	// and the case alteration should modify case post-decoding
-	if (o.hex_decode_content == 1) {  // Mod suggested by AFN Tue 8th April 2003
+	if (o.hex_decode_content) {  // Mod suggested by AFN Tue 8th April 2003
 #ifdef DGDEBUG
 		std::cout << "Hex decoding is enabled" << std::endl;
 #endif
@@ -160,7 +158,7 @@ void NaughtyFilter::checkme(DataBuffer *body, String &url, String &domain)
 #ifdef DGDEBUG
 		std::cout << "Filtering with/without case preservation is enabled" << std::endl;
 #endif
-		preserve_case = 0;
+		preserve_case = false;
 	}
 	
 	// Store for the lowercase (maybe) data
@@ -185,7 +183,7 @@ void NaughtyFilter::checkme(DataBuffer *body, String &url, String &domain)
 #endif
 		// use the one that's been hex decoded, but not stripped
 		// make a copy of the document lowercase char by char
-		if (preserve_case == 1) {
+		if (preserve_case) {
 			for (i = 0; i < hexdecodedlen; i++) {
 				c = hexdecoded[i];
 				if (c == 13 || c == 9 || c == 10) {
@@ -238,7 +236,7 @@ void NaughtyFilter::checkme(DataBuffer *body, String &url, String &domain)
 			}
 
 			// if case preserved, also look for uppercase versions
-			if ((preserve_case == 1) and (endhead == NULL)) {
+			if (preserve_case and (endhead == NULL)) {
 				endhead = strstr(bodylc, "</HEAD");
 #ifdef DGDEBUG
 				if (endhead != NULL)
@@ -266,7 +264,7 @@ void NaughtyFilter::checkme(DataBuffer *body, String &url, String &domain)
 				c = bodylc[i];
 				// are we at the start of a tag?
 				if ((!addit) && (c == '<')) {
-					if ((strncmp(bodylc+i+1, "meta", 4) == 0) or ((preserve_case == 1) and (strncmp(bodylc+i+1, "META", 4) == 0))) {
+					if ((strncmp(bodylc+i+1, "meta", 4) == 0) or (preserve_case and (strncmp(bodylc+i+1, "META", 4) == 0))) {
 #ifdef DGDEBUG
 						std::cout << "Found META" << std::endl;
 #endif
@@ -278,7 +276,7 @@ void NaughtyFilter::checkme(DataBuffer *body, String &url, String &domain)
 						c = bodylc[i];
 					}
 					// are we at the start of a title tag?
-					else if ((strncmp(bodylc+i+1, "title", 5) == 0) or ((preserve_case == 1) and (strncmp(bodylc+i+1, "TITLE", 5) == 0))) {
+					else if ((strncmp(bodylc+i+1, "title", 5) == 0) or (preserve_case and (strncmp(bodylc+i+1, "TITLE", 5) == 0))) {
 #ifdef DGDEBUG
 						std::cout << "Found TITLE" << std::endl;
 #endif
@@ -398,7 +396,7 @@ void NaughtyFilter::checkme(DataBuffer *body, String &url, String &domain)
 
 		// second time round the case loop (if there is a second time),
 		// do preserve case (exotic encodings)
-		preserve_case = 1;
+		preserve_case = true;
 	}
 	delete[]bodylc;
 	delete[]bodynohtml;
@@ -412,15 +410,12 @@ void NaughtyFilter::checkphrase(char *file, int l, String *url, String *domain)
 	int weighting = 0;
 	int cat;
 	std::string weightedphrase;
-	bool isfound, catfound;
 	
 	// checkme: translate this?
 	String currcat("Embedded URLs");
 
 	// found categories list & reusable iterators
-	std::deque<listent> listcategories;
-	std::deque<listent>::iterator cattop = listcategories.begin();
-	std::deque<listent>::iterator catcurrent;
+	std::map<int, listent> listcategories;
 
 	// check for embedded references to banned sites/URLs.
 	// have regexes that check for URLs in pages (look for attributes (src, href, javascript location)
@@ -443,10 +438,10 @@ void NaughtyFilter::checkphrase(char *file, int l, String *url, String *domain)
 	// if weighted phrases are enabled, and we have been passed a URL and domain, and embedded URL checking is enabled...
 	// then check for embedded URLs!
 	if (url != NULL && o.fg[filtergroup]->embedded_url_weight > 0) {
-		listent *ourcat = NULL;
-		std::deque<String> found;
-		std::deque<String>::iterator foundtop = found.begin();
-		std::deque<String>::iterator foundcurrent;
+		std::map<int, listent>::iterator ourcat;
+		bool catinited = false;
+		std::map<String, unsigned int> found;
+		std::map<String, unsigned int>::iterator founditem;
 
 		String u;
 		char* j;
@@ -472,31 +467,23 @@ void NaughtyFilter::checkphrase(char *file, int l, String *url, String *domain)
 					// but because inBanned* methods do some cleaning up of their own, we don't know the form to check against.
 					// we actually want these cleanups do be done before passing to inBanned*/inException* - this would
 					// speed up ConnectionHandler a bit too.
-					isfound = false;
-					if (o.weighted_phrase_mode == 2) {
-						foundcurrent = foundtop;
-						while (foundcurrent != found.end()) {
-							if (*foundcurrent == j) {
-								isfound = true;
-								break;
-							}
-							foundcurrent++;
-						}
-					}
-					if (!isfound) {
+					founditem = found.find(j);
+					if ((o.weighted_phrase_mode == 2) && (founditem != found.end())) {
+						founditem->second++;
+					} else {
 						// add the site to the found phrases list
+						found[j] = 1;
 						if (weightedphrase.length() == 0)
 							weightedphrase = "[";
 						else
 							weightedphrase += " ";
 						weightedphrase += j;
-						if (ourcat == NULL) {
-							listcategories.push_back(listent(-1,o.fg[filtergroup]->embedded_url_weight,currcat));
-							ourcat = &(listcategories.back());
+						if (!catinited) {
+							listcategories[-1] = listent(o.fg[filtergroup]->embedded_url_weight,currcat);
+							ourcat = listcategories.find(-1);
+							catinited = true;
 						} else
-							ourcat->weight += o.fg[filtergroup]->embedded_url_weight;
-						if (o.weighted_phrase_mode == 2)
-							found.push_back(j);
+							ourcat->second.weight += o.fg[filtergroup]->embedded_url_weight;
 					}
 				}
 			}
@@ -550,41 +537,33 @@ void NaughtyFilter::checkphrase(char *file, int l, String *url, String *domain)
 					// but because inBanned* methods do some cleaning up of their own, we don't know the form to check against.
 					// we actually want these cleanups do be done before passing to inBanned*/inException* - this would
 					// speed up ConnectionHandler a bit too.
-					isfound = false;
-					if (o.weighted_phrase_mode == 2) {
-						foundcurrent = foundtop;
-						while (foundcurrent != found.end()) {
-							if (*foundcurrent == j) {
-								isfound = true;
-								break;
-							}
-							foundcurrent++;
-						}
-					}
-					if (!isfound) {
+					founditem = found.find(j);
+					if ((o.weighted_phrase_mode == 2) && (founditem != found.end())) {
+						founditem->second++;
+					} else {
 						// add the site to the found phrases list
+						found[j] = 1;
 						if (weightedphrase.length() == 0)
 							weightedphrase = "[";
 						else
 							weightedphrase += " ";
 						weightedphrase += j;
-						if (ourcat == NULL) {
-							listcategories.push_back(listent(-1,o.fg[filtergroup]->embedded_url_weight,currcat));
-							ourcat = &(listcategories.back());
+						if (!catinited) {
+							listcategories[-1] = listent(o.fg[filtergroup]->embedded_url_weight,currcat);
+							ourcat = listcategories.find(-1);
+							catinited = true;
 						} else
-							ourcat->weight += o.fg[filtergroup]->embedded_url_weight;
-						if (o.weighted_phrase_mode == 2)
-							found.push_back(j);
+							ourcat->second.weight += o.fg[filtergroup]->embedded_url_weight;
 					}
 				}
 			}
 		}
-		if (ourcat != NULL) {
-			weighting = ourcat->weight;
+		if (catinited) {
+			weighting = ourcat->second.weight;
 			weightedphrase += "]";
 #ifdef DGDEBUG
 			std::cout << weightedphrase << std::endl;
-			std::cout << "score from embedded URLs: " << ourcat->weight << std::endl;
+			std::cout << "score from embedded URLs: " << ourcat->second.weight << std::endl;
 #endif
 		}
 	}
@@ -593,27 +572,18 @@ void NaughtyFilter::checkphrase(char *file, int l, String *url, String *domain)
 	std::string bannedphrase;
 	std::string exceptionphrase;
 	String bannedcategory;
-	int type, index, weight;
+	int type, index, weight, time;
 	bool allcmatched = true, bannedcombi = false;
 	std::string s1;
 
 	// this line here searches for phrases contained in the list - the rest of the code is all sorting
 	// through it to find the categories, weightings, types etc. of what has actually been found.
-	std::deque<unsigned int> found;
+	std::map<std::string, std::pair<unsigned int, unsigned int> > found;
 	(*o.lm.l[(*o.fg[filtergroup]).banned_phrase_list]).graphSearch(found, file, l);
 
 	// cache reusable iterators
-	std::deque<unsigned int>::iterator foundtop = found.begin();
-	std::deque<unsigned int>::iterator foundend = found.end();
-	std::deque<unsigned int>::iterator foundcurrent;
-	std::deque<unsigned int>::iterator alreadyfound;
-
-	// de-duplicate list of phrases if in weighted phrase mode 2
-	if (o.weighted_phrase_mode == 2) {
-		std::sort(foundtop, foundend);
-		foundend = std::unique(foundtop, foundend);
-		found.erase(foundend, found.end());
-	}
+	std::map<std::string, std::pair<unsigned int, unsigned int> >::iterator foundend = found.end();
+	std::map<std::string, std::pair<unsigned int, unsigned int> >::iterator foundcurrent;
 
 	// look for combinations first
 	//if banned must wait for exception later
@@ -621,42 +591,57 @@ void NaughtyFilter::checkphrase(char *file, int l, String *url, String *domain)
 	std::string combisofar;
 
 	std::vector<int>::iterator combicurrent = o.lm.l[o.fg[filtergroup]->banned_phrase_list]->combilist.begin();
+	std::map<int, listent>::iterator catcurrent;
+	unsigned int lowest_occurrences = 0;
 
 	while (combicurrent != o.lm.l[o.fg[filtergroup]->banned_phrase_list]->combilist.end()) {
+		// Grab the current combination phrase part
 		index = *combicurrent;
+		// Do stuff if what we have is an end marker (end of one list of parts)
 		if (index == -2) {
+			// Were all the parts in this combination matched?
 			if (allcmatched) {
 				type = *(++combicurrent);
-				if (type == -1) {	// combination exception
+				// check this time limit against the list of time limits
+				time = *(++combicurrent);
+				if (not (o.lm.l[o.fg[filtergroup]->banned_phrase_list]->checkTimeAtD(time))) {
+					// nope - so don't take any notice of it
+#ifdef DGDEBUG
+					combicurrent++;
+					cat = (*++combicurrent);
+					std::cout << "Ignoring combi phrase based on time limits: " << combisofar << "; "
+						<< o.lm.l[o.fg[filtergroup]->banned_phrase_list]->getListCategoryAtD(cat) << std::endl;
+#else
+					combicurrent += 2;
+#endif
+					combisofar = "";
+				}
+				else if (type == -1) {	// combination exception
 					isItNaughty = false;
 					isException = true;
 					whatIsNaughtyLog = o.language_list.getTranslation(605);
 					// Combination exception phrase found:
 					whatIsNaughtyLog += combisofar;
 					whatIsNaughty = "";
+					++combicurrent;
+					cat = *(++combicurrent);
+					whatIsNaughtyCategories = o.lm.l[o.fg[filtergroup]->banned_phrase_list]->getListCategoryAtD(cat);
 					return;
 				}
 				else if (type == 1) {	// combination weighting
 					weight = *(++combicurrent);
-					weighting += weight;
+					weighting += weight * (o.weighted_phrase_mode == 2 ? 1 : lowest_occurrences);
 					if (weight > 0) {
 						cat = *(++combicurrent);
 						//category index -1 indicates an uncategorised list
 						if (cat >= 0) {
 							//don't output duplicate categories
-							catcurrent = cattop;
-							catfound = false;
-							while (catcurrent != listcategories.end()) {
-								if (catcurrent->cat == cat) {
-									catfound = true;
-									catcurrent->weight += weight;
-									break;
-								}
-								catcurrent++;
-							}
-							if (!catfound) {
+							catcurrent = listcategories.find(cat);
+							if (catcurrent != listcategories.end()) {
+								catcurrent->second.weight += weight * (o.weighted_phrase_mode == 2 ? 1 : lowest_occurrences);
+							} else {
 								currcat = o.lm.l[o.fg[filtergroup]->banned_phrase_list]->getListCategoryAtD(cat);
-								listcategories.push_back(listent(cat,weight,currcat));
+								listcategories[cat] = listent(weight,currcat);
 							}
 						}
 					} else {
@@ -672,33 +657,38 @@ void NaughtyFilter::checkphrase(char *file, int l, String *url, String *domain)
 					} else {
 						weightedphrase += combisofar;
 					}
+#ifdef DGDEBUG
+					std::cout << "found combi weighted phrase ("<< o.weighted_phrase_mode << "): "
+						<< combisofar << " x" << lowest_occurrences << " (per phrase: "
+						<< weight << ", calculated: "
+						<< (weight * (o.weighted_phrase_mode == 2 ? 1 : lowest_occurrences)) << ")"
+						<< std::endl;
+#endif
 
 					weightedphrase += ")";
 					combisofar = "";
 				}
 				else if (type == 0) {	// combination banned
 					bannedcombi = true;
-					combifound = combisofar;
+					combifound += "(" + combisofar + ")";
+					combisofar = "";
 					combicurrent += 2;
 					cat = *(combicurrent);
 					bannedcategory = o.lm.l[o.fg[filtergroup]->banned_phrase_list]->getListCategoryAtD(cat);
 				}
 			} else {
+				// We had an end marker, but not all the parts so far were matched.
+				// Reset the match flag ready for the next chain, and advance to its first part.
 				allcmatched = true;
-				combicurrent += 3;
+				combicurrent += 4;
+				lowest_occurrences = 0;
 			}
 		} else {
+			// We didn't get an end marker - just an individual part.
+			// If all parts in the current chain have been matched so far, look for this one as well.
 			if (allcmatched) {
-				isfound = false;
 				s1 =(*o.lm.l[(*o.fg[filtergroup]).banned_phrase_list]).getItemAtInt(index);
-				foundcurrent = foundtop;
-				while (foundcurrent != foundend) {
-					if (s1 == (*o.lm.l[(*o.fg[filtergroup]).banned_phrase_list]).getItemAtInt(*(foundcurrent++))) {
-						isfound = true;
-						break;
-					}
-				}
-				if (!isfound) {
+				if ((foundcurrent = found.find(s1)) == foundend) {
 					allcmatched = false;
 					combisofar = "";
 				} else {
@@ -706,50 +696,62 @@ void NaughtyFilter::checkphrase(char *file, int l, String *url, String *domain)
 						combisofar += ", ";
 					}
 					combisofar += s1;
+					// also track lowest number of times any one part occurs in the text
+					// as this will correspond to the number of times the whole chain occurs
+					if ((lowest_occurrences == 0) || (lowest_occurrences > foundcurrent->second.second)) {
+						lowest_occurrences = foundcurrent->second.second;
+					}
 				}
 			}
 		}
+		// Advance to the next part in the current chain
 		combicurrent++;
 	}
 
 	// even if we already found a combi ban, we must still wait; there may be non-combi exceptions to follow
 
 	// now check non-combi phrases
-	foundcurrent = foundtop;
+	foundcurrent = found.begin();
 	while (foundcurrent != foundend) {
-		type = (*o.lm.l[(*o.fg[filtergroup]).banned_phrase_list]).getTypeAt(*foundcurrent);
+		// check time for current phrase
+		if (not o.lm.l[o.fg[filtergroup]->banned_phrase_list]->checkTimeAt(foundcurrent->second.first)) {
+#ifdef DGDEBUG
+			std::cout << "Ignoring phrase based on time limits: "
+				<< foundcurrent->first << ", "
+				<< (*o.lm.l[(*o.fg[filtergroup]).banned_phrase_list]).getListCategoryAt(foundcurrent->second.first) << std::endl;
+#endif
+			foundcurrent++;
+			continue;
+		}
 		// 0=banned, 1=weighted, -1=exception, 2=combi, 3=weightedcombi
+		type = (*o.lm.l[(*o.fg[filtergroup]).banned_phrase_list]).getTypeAt(foundcurrent->second.first);
 		if (type == 0) {
 			// if we already found a combi ban, we don't need to know this stuff
 			if (!bannedcombi) {
 				isItNaughty = true;
-				bannedphrase = (*o.lm.l[(*o.fg[filtergroup]).banned_phrase_list]).getItemAtInt(*foundcurrent);
-				bannedcategory = (*o.lm.l[(*o.fg[filtergroup]).banned_phrase_list]).getListCategoryAt(*foundcurrent, &cat);
+				bannedphrase = foundcurrent->first;
+				bannedcategory = (*o.lm.l[(*o.fg[filtergroup]).banned_phrase_list]).getListCategoryAt(foundcurrent->second.first, &cat);
 			}
 		}
 		else if (type == 1) {
-			weight = (*o.lm.l[(*o.fg[filtergroup]).banned_phrase_list]).getWeightAt(*foundcurrent);
+			// found a weighted phrase - either add one lot of its score, or one lot for every occurrence, depending on phrase filtering mode
+			weight = (*o.lm.l[(*o.fg[filtergroup]).banned_phrase_list]).getWeightAt(foundcurrent->second.first) * (o.weighted_phrase_mode == 2 ? 1 : foundcurrent->second.second);
 			weighting += weight;
 			if (weight > 0) {
-				currcat = (*o.lm.l[(*o.fg[filtergroup]).banned_phrase_list]).getListCategoryAt(*foundcurrent, &cat);
+				currcat = (*o.lm.l[(*o.fg[filtergroup]).banned_phrase_list]).getListCategoryAt(foundcurrent->second.first, &cat);
 				if (cat >= 0) {
 					//don't output duplicate categories
-					catcurrent = cattop;
-					catfound = false;
-					while (catcurrent != listcategories.end()) {
-						if (catcurrent->cat == cat) {
-							catfound = true;
-							catcurrent->weight += weight;
-							break;
-						}
-						catcurrent++;
+					catcurrent = listcategories.find(cat);
+					if (catcurrent != listcategories.end()) {
+						// add one or N times the weight to this category's score
+						catcurrent->second.weight += weight * (o.weighted_phrase_mode == 2 ? 1 : foundcurrent->second.second);
+					} else {
+						listcategories[cat] = listent(weight,currcat);
 					}
-					if (!catfound)
-						listcategories.push_back(listent(cat,weight,currcat));
 				}
 			}
 
-			if (o.show_weighted_found == 1) {
+			if (o.show_weighted_found) {
 				if (weightedphrase.length() > 0) {
 					weightedphrase += "+";
 				}
@@ -757,11 +759,13 @@ void NaughtyFilter::checkphrase(char *file, int l, String *url, String *domain)
 					weightedphrase += "-";
 				}
 
-				weightedphrase += (*o.lm.l[(*o.fg[filtergroup]).banned_phrase_list]).getItemAtInt(*foundcurrent);
+				weightedphrase += foundcurrent->first;
 			}
 #ifdef DGDEBUG
-			std::cout << "found weighted phrase ("<< o.weighted_phrase_mode << "):"
-				<< (*o.lm.l[(*o.fg[filtergroup]).banned_phrase_list]).getItemAtInt(*foundcurrent) << std::endl;
+			std::cout << "found weighted phrase ("<< o.weighted_phrase_mode << "): "
+				<< foundcurrent->first << " x" << foundcurrent->second.second << " (per phrase: "
+				<< o.lm.l[o.fg[filtergroup]->banned_phrase_list]->getWeightAt(foundcurrent->second.first)
+				<< ", calculated: " << weight << ")" << std::endl;
 #endif
 		}
 		else if (type == -1) {
@@ -769,8 +773,9 @@ void NaughtyFilter::checkphrase(char *file, int l, String *url, String *domain)
 			isItNaughty = false;
 			whatIsNaughtyLog = o.language_list.getTranslation(604);
 			// Exception phrase found:
-			whatIsNaughtyLog += (*o.lm.l[(*o.fg[filtergroup]).banned_phrase_list]).getItemAtInt(*foundcurrent);
+			whatIsNaughtyLog += foundcurrent->first;
 			whatIsNaughty = "";
+			whatIsNaughtyCategories = o.lm.l[o.fg[filtergroup]->banned_phrase_list]->getListCategoryAt(foundcurrent->second.first, NULL);
 			return;  // no point in going further
 		}
 		foundcurrent++;
@@ -819,7 +824,7 @@ void NaughtyFilter::checkphrase(char *file, int l, String *url, String *domain)
 		whatIsNaughtyLog += String((*o.fg[filtergroup]).naughtyness_limit).toCharArray();
 		whatIsNaughtyLog += " : ";
 		whatIsNaughtyLog += String(weighting).toCharArray();
-		if (o.show_weighted_found == 1) {
+		if (o.show_weighted_found) {
 			whatIsNaughtyLog += " (";
 			whatIsNaughtyLog += weightedphrase;
 			whatIsNaughtyLog += ")";
@@ -830,9 +835,15 @@ void NaughtyFilter::checkphrase(char *file, int l, String *url, String *domain)
 		bool nonempty = false;
 		bool belowthreshold = false;
 		String categories;
-		std::sort(listcategories.begin(), listcategories.end());
-		std::deque<listent>::iterator k = listcategories.begin();
-		while (k != listcategories.end()) {
+		std::deque<listent> sortable_listcategories;
+		catcurrent = listcategories.begin();
+		while (catcurrent != listcategories.end()) {
+			sortable_listcategories.push_back(catcurrent->second);
+			catcurrent++;
+		}
+		std::sort(sortable_listcategories.begin(), sortable_listcategories.end());
+		std::deque<listent>::iterator k = sortable_listcategories.begin();
+		while (k != sortable_listcategories.end()) {
 			// if category display threshold is in use, apply it
 			if (!belowthreshold && (o.fg[filtergroup]->category_threshold > 0)
 				&& (k->weight < o.fg[filtergroup]->category_threshold))
